from leapp.models import DaemonList, DistributionSignedRPM, RPM, SendmailMigrationDecision, TcpWrappersFacts
from leapp.reporting import Report
from leapp.utils.report import is_inhibitor

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'

with_sendmail = [
    RPM(name='grep', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'),
    RPM(name='sendmail', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')]

without_sendmail = [
    RPM(name='grep', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'),
    RPM(name='sed', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')]


def create_modulesfacts(installed_rpm):
    return DistributionSignedRPM(items=installed_rpm)


def test_actor_without_sendmail_package(current_actor_context):
    tcpwrap_facts = TcpWrappersFacts(daemon_lists=[])
    current_actor_context.feed(create_modulesfacts(installed_rpm=without_sendmail))
    current_actor_context.feed(tcpwrap_facts)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_actor_with_tcp_wrappers(current_actor_context):
    tcpwrap_facts = TcpWrappersFacts(daemon_lists=[DaemonList(value=['sendmail'])])
    current_actor_context.feed(create_modulesfacts(installed_rpm=with_sendmail))
    current_actor_context.feed(tcpwrap_facts)
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)



from leapp.actors import Actor
from leapp.libraries.actor import checkinstalledkernels
from leapp.models import DistributionSignedRPM, KernelInfo
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckInstalledKernels(Actor):
    """
    Inhibit IPU (in-place upgrade) when installed kernels conflict with a safe upgrade.

    a) Inhibit when multiple kernels are installed on a s390x machine

    When on s390x architecture, we are not able to upgrade correctly
    when any kernel is expected to be uninstalled during the rpm
    upgrade transaction now. We discovered recently that removal of
    old kernels is not handled correctly during the IPU. In case the
    maximum number of kernels are installed, the oldest one is
    automatically uninstalled during the rpm upgrade transaction.

    To prevent any related troubles during the IPU, inhibit the IPU
    on s390x unless just one kernel is installed, until the issue will
    be fixed correctly.

    b) Inhibit when machine is not booted into latest installed kernel

    It is strictly required that during the upgrade the machine is
    booted into the latest installed kernel. Upgrading with older
    kernels could cause unexpected issues.
    """

    name = 'check_installed_kernels'
    consumes = (DistributionSignedRPM, KernelInfo)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        checkinstalledkernels.process()



import os

import pytest

from leapp.libraries import stdlib
from leapp.libraries.actor import pluginscanner, scanpkgmanager
from leapp.libraries.common import testutils
from leapp.libraries.common.testutils import CurrentActorMocked, produce_mocked
from leapp.libraries.stdlib import api

CUR_DIR = os.path.dirname(os.path.abspath(__file__))
PROXY_ADDRESS = 'https://192.168.121.123:3128'
YUM_CONFIG_PATH = '/etc/yum.conf'
DNF_CONFIG_PATH = '/etc/dnf/dnf.conf'


def mock_releasever_exists(overrides):
    def mocked_releasever_exists(name):
        if name in overrides:
            return overrides[name]
        raise ValueError
    return mocked_releasever_exists


def mocked_get_releasever_path():
    return os.path.join(CUR_DIR, 'files/releasever')


@pytest.mark.parametrize('etcrelease_exists', [True, False])
def test_get_etcreleasever(monkeypatch, etcrelease_exists):
    monkeypatch.setattr(
        scanpkgmanager,
        '_releasever_exists', mock_releasever_exists(
            {
                os.path.join(CUR_DIR, 'files/releasever'): etcrelease_exists,
            }
        )
    )
    monkeypatch.setattr(scanpkgmanager.api, 'produce', produce_mocked())
    monkeypatch.setattr(scanpkgmanager.api, 'current_actor', CurrentActorMocked())
    monkeypatch.setattr(scanpkgmanager, '_get_releasever_path', mocked_get_releasever_path)
    monkeypatch.setattr(scanpkgmanager, '_get_proxy_if_set', lambda x: None)
    monkeypatch.setattr(pluginscanner, 'scan_enabled_package_manager_plugins', lambda: [])

    scanpkgmanager.process()

    assert scanpkgmanager.api.produce.called
    if etcrelease_exists:
        assert api.produce.model_instances[0].etc_releasever
    else:
        assert not api.produce.model_instances[0].etc_releasever


@pytest.mark.parametrize('proxy_set', [True, False])
def test_get_proxy_if_set(monkeypatch, proxy_set):

    config_path = '/path/to/config.conf'
    config_contents = '[main]\n'
    if proxy_set:
        config_contents += 'proxy = \t{} '.format(PROXY_ADDRESS)

    def mocked_get_config_contents(path):
        assert path == config_path
        return config_contents

    monkeypatch.setattr(scanpkgmanager, '_get_config_contents', mocked_get_config_contents)

    proxy = scanpkgmanager._get_proxy_if_set(config_path)

    if proxy_set:
        assert proxy == PROXY_ADDRESS

    assert proxy_set == bool(proxy)


@pytest.mark.parametrize(
    ('proxy_set_in_dnf_config', 'proxy_set_in_yum_config', 'expected_output'),
    [
      (True, True, [PROXY_ADDRESS]),
      (True, False, [PROXY_ADDRESS]),
      (False, False, [])
    ]
)
def test_get_configured_proxies(monkeypatch, proxy_set_in_dnf_config, proxy_set_in_yum_config, expected_output):

    def mocked_get_proxy_if_set(path):
        proxy = PROXY_ADDRESS if proxy_set_in_yum_config else None
        if path == DNF_CONFIG_PATH:
            proxy = PROXY_ADDRESS if proxy_set_in_dnf_config else None
        return proxy

    monkeypatch.setattr(scanpkgmanager, '_get_proxy_if_set', mocked_get_proxy_if_set)

    configured_proxies = scanpkgmanager.get_configured_proxies()
    assert configured_proxies == expected_output



from leapp.actors import Actor
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import SatelliteFacts
from leapp.tags import FirstBootPhaseTag, IPUWorkflowTag


class SatelliteUpgrader(Actor):
    """
    Execute installer in the freshly booted system, to finalize Satellite configuration
    """

    name = 'satellite_upgrader'
    consumes = (SatelliteFacts, )
    produces = ()
    tags = (IPUWorkflowTag, FirstBootPhaseTag)

    def process(self):
        facts = next(self.consume(SatelliteFacts), None)
        if not facts or not facts.has_foreman:
            return

        if facts.postgresql.local_postgresql:
            api.current_actor().show_message('Re-indexing the database. This can take a while.')
            try:
                run(['sed', '-i', '/data_directory/d', '/var/lib/pgsql/data/postgresql.conf'])
                run(['systemctl', 'start', 'postgresql'])
                run(['runuser', '-u', 'postgres', '--', 'reindexdb', '-a'])
            except CalledProcessError as e:
                api.current_logger().error('Failed to reindex the database: {}'.format(str(e)))

        installer_cmd = ['foreman-installer']
        if facts.has_katello_installer:
            installer_cmd.append('--disable-system-checks')

        api.current_actor().show_message('Running the installer. This can take a while.')
        try:
            run(installer_cmd)
        except OSError as e:
            api.current_logger().error('Failed to run `foreman-installer`: {}'.format(str(e)))
        except CalledProcessError:
            api.current_logger().error(
                'Could not run the installer, please inspect the logs in /var/log/foreman-installer!'
            )



from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import systemd
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import SystemdBrokenSymlinksSource, SystemdServicesInfoSource, SystemdServicesPresetInfoSource


def scan():
    try:
        broken_symlinks = systemd.get_broken_symlinks()
    except (OSError, CalledProcessError) as err:
        details = {'details': str(err)}
        if isinstance(err, CalledProcessError):
            details['stderr'] = err.stderr
        raise StopActorExecutionError(
            message='Cannot scan the system to list possible broken systemd symlinks.',
            details=details
        )

    try:
        services_files = systemd.get_service_files()
    except CalledProcessError as err:
        raise StopActorExecutionError(
            message='Cannot obtain the list of systemd service unit files.',
            details={'details': str(err), 'stderr': err.stderr}
        )

    try:
        presets = systemd.get_system_service_preset_files(services_files, ignore_invalid_entries=False)
    except (OSError, CalledProcessError) as err:
        details = {'details': str(err)}
        if isinstance(err, CalledProcessError):
            details['stderr'] = err.stderr
        raise StopActorExecutionError(
            message='Cannot obtain the list of systemd preset files.',
            details=details
        )
    except ValueError as err:
        raise StopActorExecutionError(
            message='Discovered an invalid systemd preset file.',
            details={'details': str(err)}
        )

    api.produce(SystemdBrokenSymlinksSource(broken_symlinks=broken_symlinks))
    api.produce(SystemdServicesInfoSource(service_files=services_files))
    api.produce(SystemdServicesPresetInfoSource(presets=presets))



from __future__ import print_function

import json
import sys

from leapp.cli.commands.upgrade.util import fetch_all_upgrade_contexts
from leapp.exceptions import CommandError
from leapp.utils.clicmd import command


@command('list-runs', help='List previous Leapp upgrade executions')
def list_runs(args):  # noqa; pylint: disable=unused-argument
    contexts = fetch_all_upgrade_contexts()
    if contexts:
        for context in contexts:
            print('Context ID: {} - time: {} - details: {}'.format(context[0], context[1], json.loads(context[2])),
                  file=sys.stdout)
    else:
        raise CommandError('No previous run found!')


def register(base_command):
    """
        Registers `leapp register`
    """
    base_command.add_sub(list_runs)



import os

import pytest

from leapp.libraries.common import mdraid
from leapp.libraries.common.testutils import logger_mocked
from leapp.libraries.stdlib import api, CalledProcessError

MD_DEVICE = '/dev/md0'
NOT_MD_DEVICE = '/dev/sda'

CUR_DIR = os.path.dirname(os.path.abspath(__file__))


def raise_call_error(args=None):
    raise CalledProcessError(
        message='A Leapp Command Error occurred.',
        command=args,
        result={'signal': None, 'exit_code': 1, 'pid': 0, 'stdout': 'fake', 'stderr': 'fake'}
    )


class RunMocked(object):

    def __init__(self, raise_err=False):
        self.called = 0
        self.args = None
        self.raise_err = raise_err

    def __call__(self, args, encoding=None):
        self.called += 1
        self.args = args
        if self.raise_err:
            raise_call_error(args)

        if self.args == ['mdadm', '--query', MD_DEVICE]:
            stdout = '/dev/md0: 1022.00MiB raid1 2 devices, 0 spares. Use mdadm --detail for more detail.'
        elif self.args == ['mdadm', '--query', NOT_MD_DEVICE]:
            stdout = '/dev/sda: is not an md array'

        elif self.args == ['mdadm', '--detail', '--verbose', '--brief', MD_DEVICE]:
            stdout = 'ARRAY /dev/md0 level=raid1 num-devices=2 metadata=1.2 name=localhost.localdomain:0 UUID=c4acea6e:d56e1598:91822e3f:fb26832c\n    devices=/dev/sda1,/dev/sdb1'  # noqa: E501; pylint: disable=line-too-long
        elif self.args == ['mdadm', '--detail', '--verbose', '--brief', NOT_MD_DEVICE]:
            stdout = 'mdadm: /dev/sda does not appear to be an md device'

        return {'stdout': stdout}


@pytest.mark.parametrize('dev,expected', [(MD_DEVICE, True), (NOT_MD_DEVICE, False)])
def test_is_mdraid_dev(monkeypatch, dev, expected):
    run_mocked = RunMocked()
    monkeypatch.setattr(mdraid, 'run', run_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(os.path, 'exists', lambda dummy: True)

    result = mdraid.is_mdraid_dev(dev)
    assert mdraid.run.called == 1
    assert expected == result
    assert not api.current_logger.warnmsg


def test_is_mdraid_dev_error(monkeypatch):
    run_mocked = RunMocked(raise_err=True)
    monkeypatch.setattr(mdraid, 'run', run_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(os.path, 'exists', lambda dummy: True)

    with pytest.raises(CalledProcessError) as err:
        mdraid.is_mdraid_dev(MD_DEVICE)

    assert mdraid.run.called == 1
    expect_msg = 'Could not check if device "{}" is an md device:'.format(MD_DEVICE)
    assert expect_msg in err.value.message


def test_is_mdraid_dev_notool(monkeypatch):
    run_mocked = RunMocked(raise_err=True)
    monkeypatch.setattr(mdraid, 'run', run_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(os.path, 'exists', lambda dummy: False)

    result = mdraid.is_mdraid_dev(MD_DEVICE)
    assert not result
    assert not mdraid.run.called
    assert api.current_logger.warnmsg


def test_get_component_devices_ok(monkeypatch):
    run_mocked = RunMocked()
    monkeypatch.setattr(mdraid, 'run', run_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    result = mdraid.get_component_devices(MD_DEVICE)
    assert mdraid.run.called == 1
    assert ['/dev/sda1', '/dev/sdb1'] == result
    assert not api.current_logger.warnmsg


def test_get_component_devices_not_md_device(monkeypatch):
    run_mocked = RunMocked()
    monkeypatch.setattr(mdraid, 'run', run_mocked)

    with pytest.raises(ValueError) as err:
        mdraid.get_component_devices(NOT_MD_DEVICE)

    assert mdraid.run.called == 1
    expect_msg = 'Expected md device, but got: {}'.format(NOT_MD_DEVICE)
    assert expect_msg in str(err.value)



from leapp.actors import Actor
from leapp.libraries.actor import scanner
from leapp.models import GrubConfigError
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class ScanGrubConfig(Actor):
    """
    Scan grub configuration files for errors.
    """

    name = 'scan_grub_config'
    consumes = ()
    produces = (GrubConfigError,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        errors = scanner.scan()
        if errors:
            for error in errors:
                self.produce(error)



import os

from leapp.libraries.common.config.version import get_target_major_version
from leapp.libraries.stdlib import api
from leapp.models import CustomTargetRepositoryFile, TargetOSInstallationImage


def produce_repofile_if_iso_used():
    target_iso_msgs_iter = api.consume(TargetOSInstallationImage)
    target_iso = next(target_iso_msgs_iter, None)

    if not target_iso:
        return

    if next(target_iso_msgs_iter, None):
        api.current_logger().warning('Received multiple TargetISInstallationImage messages, using the first one')

    # Mounting was successful, create a repofile to copy into target userspace
    repofile_entry_template = ('[{repoid}]\n'
                               'name={reponame}\n'
                               'baseurl={baseurl}\n'
                               'enabled=0\n'
                               'gpgcheck=0\n')

    repofile_content = ''
    for repo in target_iso.repositories:
        repofile_content += repofile_entry_template.format(repoid=repo.repoid,
                                                           reponame=repo.repoid,
                                                           baseurl=repo.baseurl)

    target_os_path_prefix = 'el{target_major_ver}'.format(target_major_ver=get_target_major_version())
    iso_repofile_path = os.path.join('/var/lib/leapp/', '{}_iso.repo'.format(target_os_path_prefix))
    with open(iso_repofile_path, 'w') as iso_repofile:
        iso_repofile.write(repofile_content)

    api.produce(CustomTargetRepositoryFile(file=iso_repofile_path))



import pytest

from leapp import reporting
from leapp.libraries.actor import check_consumed_assets as check_consumed_assets_lib
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import ConsumedDataAsset
from leapp.utils.report import is_inhibitor


@pytest.mark.parametrize(('asset_data_streams', 'inhibit_reason'),
                         ((['10.0'], None),
                          (['9.3', '10.1', '11.0'], None),
                          (['11.1'], 'incompatible'),
                          (['3.1', '4.0'], 'incompatible'),
                          (['11.1', '12.0'], 'incompatible'),
                          ([], 'malformed'),
                          (['malformed'], 'malformed')))
def test_asset_version_correctness_assessment(monkeypatch, asset_data_streams, inhibit_reason):

    monkeypatch.setattr(check_consumed_assets_lib, 'get_consumed_data_stream_id', lambda: '10.0')
    used_asset = ConsumedDataAsset(filename='asset.json',
                                   fulltext_name='',
                                   docs_url='',
                                   docs_title='',
                                   provided_data_streams=asset_data_streams)

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[used_asset]))
    create_report_mock = create_report_mocked()
    monkeypatch.setattr(reporting, 'create_report', create_report_mock)

    check_consumed_assets_lib.inhibit_if_assets_with_incorrect_version()

    expected_report_count = 1 if inhibit_reason else 0
    assert create_report_mock.called == expected_report_count
    if inhibit_reason:
        report = create_report_mock.reports[0]
        assert is_inhibitor(report)
        assert inhibit_reason in report['title'].lower()


def test_make_report_entries_with_unique_urls():
    # Check that multiple titles produce one report
    docs_url_to_title_map = {'/path/to/asset1': ['asset1_title1', 'asset1_title2'],
                             '/path/to/asset2': ['asset2_title']}
    report_urls = check_consumed_assets_lib.make_report_entries_with_unique_urls(docs_url_to_title_map)
    assert set([ru.value['url'] for ru in report_urls]) == {'/path/to/asset1', '/path/to/asset2'}



from leapp.libraries.stdlib import CalledProcessError, run


def selinux_set_permissive():
    """ Set SElinux to permissive mode if it was in enforcing mode """
    cmd = ['/bin/sed', '-i', 's/^SELINUX=enforcing/SELINUX=permissive/g', '/etc/selinux/config']
    try:
        run(cmd)
    except CalledProcessError as e:
        return False, e.output
    return True, None



import logging
import sys

from leapp.repository.scan import find_and_scan_repositories

def err_exit():
    # We want to be sure that `make test` (test_no_lint) will stop when expected
    # actor is not found and want to be sure that users will not overlook error
    # messages. This print will easily resolve the problem
    sys.stdout.write('ERROR:__read_error_messages_above_this_one_on_stderr__')
    sys.exit(1)

def main():
    logging.basicConfig(level=logging.INFO, filename='/dev/null')
    logger = logging.getLogger('run_pytest.py')

    BASE_REPO = 'repos'
    repos = find_and_scan_repositories(BASE_REPO, include_locals=True)
    repos.load()
    if len(sys.argv) > 1:
        actors = repos._lookup_actors(sys.argv[1])
        if not actors:
            sys.stderr.write('ERROR: No actor found for search "{}"\n'.format(sys.argv[1]))
            err_exit()
        print(' '.join([actor.full_path for actor in actors]))
    else:
        sys.stderr.write('ERROR: Missing commandline argument\n')
        err_exit()


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        # Make a generic exception as in case of error, without the explicit
        # sys.exit(1) call running of tests continues with all actors always
        # and we expect that many people will be affected unless they installed
        sys.stderr.write('ERROR: Unknown error: {}\n'.format(e))
        sys.stderr.write('ERROR: Possibly you need newer version of the leapp framework\n')
        sys.stderr.write('ERROR:   e.g.: rm -rf .tut && make install-deps-fedora\n')
        err_exit()



from leapp import reporting
from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM, OpenSshConfig, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class OpenSshDropInDirectoryCheck(Actor):
    """
    Trigger a notice that the main sshd_config will be updated to contain
    the Include directive so the other configuration files dropped by the
    RHEL9 packages are effective.

    This might change the sshd behavior so it is advised to verify by the
    customer that the updated system behaves as expected.
    """

    name = 'open_ssh_drop_in_directory_check'
    consumes = (OpenSshConfig, DistributionSignedRPM,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag,)

    def process(self):
        openssh_messages = self.consume(OpenSshConfig)
        config = next(openssh_messages, None)
        if list(openssh_messages):
            api.current_logger().warning('Unexpectedly received more than one OpenSshConfig message.')
        if not config:
            raise StopActorExecutionError(
                'Could not check openssh configuration', details={'details': 'No OpenSshConfig facts found.'}
            )

        # If the package is not installed, there is no need to do anything
        if not has_package(DistributionSignedRPM, 'openssh-server'):
            return

        # If the configuration file was not modified, the rpm update will bring the new
        # changes by itself
        if not config.modified:
            return

        # otherwise we will prepend the Include directive to the main sshd_config
        resources = [
            reporting.RelatedResource('package', 'openssh-server'),
            reporting.RelatedResource('file', '/etc/ssh/sshd_config')
        ]
        reporting.create_report([
            reporting.Title('The upgrade will prepend the Include directive to OpenSSH sshd_config'),
            reporting.Summary(
                'OpenSSH server configuration needs to be modified to contain Include directive '
                'for the RHEL9 to work properly and integrate with the other parts of the OS. '
                'The following snippet will be added to the /etc/ssh/sshd_config during the '
                'ApplicationsPhase: `Include /etc/ssh/sshd_config.d/*.conf`'
            ),
            reporting.Severity(reporting.Severity.INFO),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.NETWORK,
                    reporting.Groups.SERVICES
            ]),
        ] + resources)



import glob
import os
import shutil

from leapp.actors import Actor
from leapp.models import SatelliteFacts
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag

POSTGRESQL_DATA_PATH = '/var/lib/pgsql/data/'
POSTGRESQL_SCL_DATA_PATH = '/var/opt/rh/rh-postgresql12/lib/pgsql/data/'
POSTGRESQL_USER = 'postgres'
POSTGRESQL_GROUP = 'postgres'

SYSTEMD_WANTS_BASE = '/etc/systemd/system/multi-user.target.wants/'
SERVICES_TO_DISABLE = ['dynflow-sidekiq@*', 'foreman', 'foreman-proxy',
                       'httpd', 'postgresql', 'pulpcore-api', 'pulpcore-content',
                       'pulpcore-worker@*', 'tomcat']


class SatelliteUpgradeDataMigration(Actor):
    """
    Reconfigure Satellite services and migrate PostgreSQL data
    """

    name = 'satellite_upgrade_data_migration'
    consumes = (SatelliteFacts,)
    produces = ()
    tags = (IPUWorkflowTag, ApplicationsPhaseTag)

    def process(self):
        facts = next(self.consume(SatelliteFacts), None)
        if not facts or not facts.has_foreman:
            return

        # disable services, will be re-enabled by the installer
        for service_name in SERVICES_TO_DISABLE:
            for service in glob.glob(os.path.join(SYSTEMD_WANTS_BASE, '{}.service'.format(service_name))):
                try:
                    os.unlink(service)
                except Exception as e:  # pylint: disable=broad-except
                    self.log.warning('Failed disabling service {}: {}'.format(service, e))

        if facts.postgresql.local_postgresql and os.path.exists(POSTGRESQL_SCL_DATA_PATH):
            # we can assume POSTGRESQL_DATA_PATH exists and is empty
            # move PostgreSQL data to the new home
            for item in glob.glob(os.path.join(POSTGRESQL_SCL_DATA_PATH, '*')):
                try:
                    shutil.move(item, POSTGRESQL_DATA_PATH)
                except Exception as e:  # pylint: disable=broad-except
                    self.log.warning('Failed moving PostgreSQL data: {}'.format(e))
                    return

            if not facts.postgresql.same_partition:
                for dirpath, _, filenames in os.walk(POSTGRESQL_DATA_PATH):
                    try:
                        shutil.chown(dirpath, POSTGRESQL_USER, POSTGRESQL_GROUP)
                        for filename in filenames:
                            shutil.chown(os.path.join(dirpath, filename), POSTGRESQL_USER, POSTGRESQL_GROUP)
                    except Exception as e:  # pylint: disable=broad-except
                        self.log.warning('Failed fixing ownership of PostgreSQL data: {}'.format(e))
                        return



import pytest

from leapp import reporting
from leapp.libraries.actor import rocecheck
from leapp.libraries.common.config import architecture
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import KernelCmdline, KernelCmdlineArg, RoceDetected


def _kernel_cmdline(params=None):
    if params is None:
        return KernelCmdline(parameters=[])
    k_params = []
    for item in params:
        try:
            key, value = item.split('=', 1)
        except ValueError:
            key = item
            value = None
        k_params.append(KernelCmdlineArg(key=key, value=value))
    return KernelCmdline(parameters=k_params)


def _roce(connected, connecting):
    return RoceDetected(
        roce_nics_connected=connected,
        roce_nics_connecting=connecting
    )


@pytest.mark.parametrize('msgs', (
    [_kernel_cmdline()],
    [_kernel_cmdline(), _roce([], [])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.7']), _roce([], [])],
))
def test_no_roce(monkeypatch, msgs):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=architecture.ARCH_S390X, msgs=msgs))
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    rocecheck.process()
    assert not reporting.create_report.called


@pytest.mark.parametrize('arch', (
    architecture.ARCH_ARM64,
    architecture.ARCH_X86_64,
    architecture.ARCH_PPC64LE
))
def test_roce_noibmz(monkeypatch, arch):
    def mocked_do_not_call_me(dummy):
        assert False, 'Unexpected call on non-IBMz arch (actor should not do anything).'

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=arch))
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    monkeypatch.setattr(rocecheck, '_report_old_version', mocked_do_not_call_me)
    monkeypatch.setattr(rocecheck, '_report_wrong_setup', mocked_do_not_call_me)
    monkeypatch.setattr(rocecheck, 'is_kernel_arg_set', mocked_do_not_call_me)
    monkeypatch.setattr(rocecheck.api, 'consume', mocked_do_not_call_me)
    rocecheck.process()


@pytest.mark.parametrize('msgs', (
    [_kernel_cmdline(['net.naming-scheme=rhel-8.7']), _roce(['eno'], [])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.7']), _roce([], ['eno'])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.7']), _roce(['enp0', 'enp1'], ['eno'])],
    [_kernel_cmdline(['good', 'net.naming-scheme=rhel-8.7']), _roce(['eno'], [])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.7', 'good']), _roce(['eno'], [])],
    [_kernel_cmdline(['foo=bar', 'net.naming-scheme=rhel-8.7', 'foo=bar']), _roce(['eno'], [])],
))
@pytest.mark.parametrize('version', ['8.7', '8.8', '8.10'])
def test_roce_ok(monkeypatch, msgs, version):
    curr_actor_mocked = CurrentActorMocked(arch=architecture.ARCH_S390X, src_ver=version, msgs=msgs)
    monkeypatch.setattr(api, 'current_actor', curr_actor_mocked)
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    rocecheck.process()
    assert not reporting.create_report.called


@pytest.mark.parametrize('msgs', (
    [_kernel_cmdline(['net.naming-scheme=rhel-8.7']), _roce(['eno'], [])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.7']), _roce([], ['eno'])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.6']), _roce(['eno'], [])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.6']), _roce(['eno', 'eno1'], ['enp'])],
    [_kernel_cmdline(['foo=bar']), _roce(['eno'], [])],
    [_kernel_cmdline(), _roce(['eno'], [])],
))
@pytest.mark.parametrize('version', ['8.0', '8.3', '8.6'])
def test_roce_old_rhel(monkeypatch, msgs, version):
    curr_actor_mocked = CurrentActorMocked(arch=architecture.ARCH_S390X, src_ver=version, msgs=msgs)
    monkeypatch.setattr(api, 'current_actor', curr_actor_mocked)
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    rocecheck.process()
    assert reporting.create_report.called
    assert any(['version of RHEL' in report['title'] for report in reporting.create_report.reports])


# NOTE: what about the situation when net.naming-scheme is configured multiple times???
@pytest.mark.parametrize('msgs', (
    [_kernel_cmdline(['net.naming-scheme=rhel-8.6']), _roce(['eno'], [])],
    [_kernel_cmdline(['net.naming-scheme=rhel-8.8']), _roce([], ['eno'])],
    [_kernel_cmdline(['foo=bar', 'net.naming-scheme=rhel-8.8']), _roce([], ['eno'])],
    [_kernel_cmdline(['foo=bar', 'net.naming-scheme=rhel-8.8', 'foo=bar']), _roce([], ['eno'])],
    [_kernel_cmdline(['net.naming-scheme']), _roce(['eno'], [])],
    [_kernel_cmdline(['foo=bar']), _roce(['eno'], [])],
    [_kernel_cmdline(['foo=bar', 'bar=foo']), _roce(['eno'], [])],
    [_kernel_cmdline(['rhel-8.7']), _roce([], ['eno'])],
    [_kernel_cmdline(), _roce(['eno'], [])],
))
@pytest.mark.parametrize('version', ['8.6', '8.8'])
def test_roce_wrong_configuration(monkeypatch, msgs, version):
    curr_actor_mocked = CurrentActorMocked(arch=architecture.ARCH_S390X, src_ver=version, msgs=msgs)
    monkeypatch.setattr(api, 'current_actor', curr_actor_mocked)
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    rocecheck.process()
    assert reporting.create_report.called
    assert any(['RoCE configuration' in report['title'] for report in reporting.create_report.reports])



import errno
import os

from leapp.libraries.actor import spamassassinconfigread_spamd
from leapp.libraries.common.testutils import make_IOError, make_OSError


class MockFileOperations(object):
    def __init__(self, to_raise=None):
        self.files = {}
        self.files_read = {}
        self.read_called = 0
        self.to_raise = to_raise

    def _increment_read_counters(self, path):
        self.read_called += 1
        self.files_read.setdefault(path, 0)
        self.files_read[path] += 1

    def read(self, path):
        self._increment_read_counters(path)
        if self.to_raise is not None:
            raise self.to_raise
        try:
            return self.files[path]
        except KeyError:
            raise make_IOError(errno.ENOENT)


class MockListDir(object):
    def __init__(self, path=None, file_names=None, to_raise=None):
        self.path = None if path is None else os.path.normpath(path)
        self.file_names = file_names
        self.to_raise = to_raise
        self.error = False

    def listdir(self, path):
        if self.to_raise:
            raise self.to_raise
        if os.path.normpath(path) == self.path:
            return self.file_names
        self.error = True
        raise make_OSError(errno.ENOENT)


def test_spamassassin_service_overriden():
    listdir = MockListDir(path='/etc/systemd/system', file_names=['spamassassin.service'])
    overridden = spamassassinconfigread_spamd.spamassassin_service_overriden(listdir.listdir)
    assert overridden is True

    listdir = MockListDir(path='/etc/systemd/system',
                          file_names=['foo.service', 'spamassassin.service', 'bar.service'])
    overridden = spamassassinconfigread_spamd.spamassassin_service_overriden(listdir.listdir)
    assert overridden is True
    assert not listdir.error


def test_spamassassin_service_overriden_nonexistent():
    listdir = MockListDir(path='/etc/systemd/system', file_names=[])
    overridden = spamassassinconfigread_spamd.spamassassin_service_overriden(listdir.listdir)
    assert overridden is False

    listdir = MockListDir(path='/etc/systemd/system',
                          file_names=['foo.service', 'bar.service'])
    overridden = spamassassinconfigread_spamd.spamassassin_service_overriden(listdir.listdir)
    assert overridden is False
    assert not listdir.error


def test_spamassassin_service_overriden_nonexistent_dir():
    listdir = MockListDir(to_raise=make_OSError(errno.ENOENT))
    overridden = spamassassinconfigread_spamd.spamassassin_service_overriden(listdir.listdir)
    assert overridden is False


def test_spamassassin_service_overriden_nonexistent_inaccessible():
    # If we can't check if the file is there, we treat it as if it was there,
    # so that the SpamassassinConfigUpdate actor doesn't make changes to
    # /etc/sysconfig/spamassassin that may not be justified.
    listdir = MockListDir(to_raise=make_OSError(errno.EACCES))
    overridden = spamassassinconfigread_spamd.spamassassin_service_overriden(listdir.listdir)
    assert overridden is True


def test_parse_ssl_version_sslv3():
    content = 'SPAMDOPTIONS="--ssl-version sslv3"'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'sslv3'

    content = 'SPAMDOPTIONS="-cd -m5 --ssl-version sslv3 -H"'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'sslv3'


def test_parse_ssl_version_tlsv1():
    content = 'SPAMDOPTIONS="--ssl-version tlsv1"'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'tlsv1'

    content = 'SPAMDOPTIONS="-cd -m5 --ssl-version tlsv1 -H"'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'tlsv1'


def test_parse_ssl_version_invalid_argument():
    # If an invalid argument is used, we treat it as if the option was not
    # specified at all, so that the config update actor doesn't touch it. We
    # don't want to break the config even more.
    content = 'SPAMDOPTIONS="--ssl-version foo"\n'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value is None


def test_parse_ssl_version_comments():
    content = '# foo\nSPAMDOPTIONS="--ssl-version tlsv1"\n# bar\n'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'tlsv1'


def test_parse_ssl_version_repeated():
    # The last --ssl-version option takes effect
    content = 'SPAMDOPTIONS="--ssl-version tlsv1 --ssl-version sslv3"\n'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'sslv3'

    content = 'SPAMDOPTIONS="--ssl-version sslv3 --ssl-version tlsv1"\n'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'sslv3'


def test_parse_ssl_version_last_assignment_takes_effect():
    # The last assignment to SPAMDOPTIONS takes effect
    content = 'SPAMDOPTIONS="--ssl-version tlsv1"\nSPAMDOPTIONS="--ssl-version sslv3"\n'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'sslv3'


def test_parse_ssl_version_multiline():
    content = 'SPAMDOPTIONS="--ssl \\\n --ssl-version tlsv1"\n'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'tlsv1'

    content = 'SPAMDOPTIONS="--ssl-version \\\n sslv3"\n'
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'sslv3'


def test_parse_ssl_version_multiline_comment():
    content = ('SPAMDOPTIONS="--ssl-version tlsv1"\n'
               '# foo \\\nSPAMDOPTIONS="--ssl-version sslv3" \\\n still a comment')
    value = spamassassinconfigread_spamd._parse_ssl_version(content)
    assert value == 'tlsv1'


def test_get_spamd_ssl_version():
    path = '/etc/sysconfig/spamassassin'
    fileops = MockFileOperations()
    fileops.files[path] = '# foo\nSPAMDOPTIONS="--ssl-version tlsv1"\n# bar\n'

    value = spamassassinconfigread_spamd.get_spamd_ssl_version(fileops.read)

    assert value == 'tlsv1'


def test_get_spamd_ssl_version_nonexistent():
    fileops = MockFileOperations()
    value = spamassassinconfigread_spamd.get_spamd_ssl_version(fileops.read)
    assert value is None


def test_get_spamd_ssl_version_inaccessible():
    fileops = MockFileOperations(to_raise=make_IOError(errno.EACCES))
    value = spamassassinconfigread_spamd.get_spamd_ssl_version(fileops.read)
    assert value is None



from leapp.actors import Actor
from leapp.libraries.actor import enabledeviceciofreeservice
from leapp.models import SystemdServicesTasks
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class EnableDeviceCioFreeService(Actor):
    """
    Enables device_cio_free.service systemd service on s390x

    After an upgrade this service ends up disabled even though it's vendor preset is set to enabled.
    The service is used to enable devices which are not explicitly enabled on the kernel command line.
    """

    name = 'enable_device_cio_free_service'
    consumes = ()
    produces = (SystemdServicesTasks,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        enabledeviceciofreeservice.process()



from leapp.actors import Actor
from leapp.libraries.actor import check_target_iso
from leapp.models import Report, StorageInfo, TargetOSInstallationImage
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckTargetISO(Actor):
    """
    Check that the provided target ISO is a valid ISO image and is located on a persistent partition.
    """

    name = 'check_target_iso'
    consumes = (StorageInfo, TargetOSInstallationImage,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        check_target_iso.perform_target_iso_checks()



from leapp.actors import Actor
from leapp.libraries import stdlib
from leapp.libraries.common import rhsm
from leapp.libraries.common.rpms import get_installed_rpms
from leapp.models import LeftoverPackages, RemovedPackages, RPM
from leapp.reporting import Report
from leapp.tags import ExperimentalTag, IPUWorkflowTag, RPMUpgradePhaseTag


class RemoveLeftoverPackages(Actor):
    """
    Remove el7 packages left on the system after the upgrade to RHEL 8.

    Removal of el7 packages is necessary in order to keep the machine in supported state.
    Actor generates report telling users what packages have been removed.
    """

    name = 'remove_leftover_packages'
    consumes = (LeftoverPackages, )
    produces = (Report, RemovedPackages)
    tags = (RPMUpgradePhaseTag, IPUWorkflowTag, ExperimentalTag)

    def process(self):
        leftover_packages = next(self.consume(LeftoverPackages), LeftoverPackages())
        if not leftover_packages.items:
            self.log.info('No leftover packages, skipping...')
            return

        installed_rpms = get_installed_rpms()

        to_remove = ['-'.join([pkg.name, pkg.version, pkg.release]) for pkg in leftover_packages.items]
        cmd = ['dnf', 'remove', '-y', '--noautoremove'] + to_remove
        if rhsm.skip_rhsm():
            # ensure we don't use suscription-manager when it should be skipped
            cmd += ['--disableplugin', 'subscription-manager']
        try:
            stdlib.run(cmd)
        except stdlib.CalledProcessError:
            error = 'Failed to remove packages: {}'.format(', '.join(to_remove))
            self.log.error(error)
            return

        removed_packages = RemovedPackages()
        removed = list(set(installed_rpms) - set(get_installed_rpms()))
        for pkg in removed:
            name, version, release, epoch, packager, arch, pgpsig = pkg.split('|')
            removed_packages.items.append(RPM(
                name=name,
                version=version,
                epoch=epoch,
                packager=packager,
                arch=arch,
                release=release,
                pgpsig=pgpsig
            ))
        self.produce(removed_packages)



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class KernelCmdlineArg(Model):
    """
    Single kernel command line parameter with/without a value

    When produced alone, represents a parameter to include in RHEL-8 kernel cmdline.
    """
    topic = SystemInfoTopic

    key = fields.String()
    value = fields.Nullable(fields.String())


class TargetKernelCmdlineArgTasks(Model):
    """
    Desired modifications of the target kernel args
    """
    topic = SystemInfoTopic

    to_add = fields.List(fields.Model(KernelCmdlineArg), default=[])
    to_remove = fields.List(fields.Model(KernelCmdlineArg), default=[])


class KernelCmdline(Model):
    """
    Kernel command line parameters the system was booted with
    """
    topic = SystemInfoTopic

    parameters = fields.List(fields.Model(KernelCmdlineArg))



def updateFirewallConfigCommand(root, old_command):
    changed = False

    # Only update the command element that corresponds to firewall-config
    new_command = '/usr/libexec/platform-python -s /usr/bin/firewall-config'
    for command in root.iter('command'):
        if 'name' in command.attrib and \
           old_command == command.attrib['name'] and \
           old_command != new_command:
            command.attrib['name'] = new_command
            changed = True

    return changed



import pytest
from six import text_type

from leapp.libraries.actor import checkbrltty

BRLTTY_CONF = 'brltty.conf'


@pytest.mark.parametrize('test_input,expected_migrate_bt,expected_migrate_espeak', [
    ('braille-device serial:/dev/ttyS0\n', False, False),
    ('braille-device bth:AB-cd:ef:01:23:45\n', True, False),
    ('braille-device bluez:AB-cd:ef:01:23:45\n', True, False),
    ('speech-driver es\n', False, True),
    ('braille-device bth:AB-cd:ef:01:23:45\nbraille-device bluez:AB-cd:ef:01:23:45\nspeech-driver es\n', True, True),
])
def test_actor_check_migration_bth(tmpdir, monkeypatch, test_input, expected_migrate_bt, expected_migrate_espeak,
                                   current_actor_context):
    test_cfg_file = text_type(tmpdir.join(BRLTTY_CONF))
    with open(test_cfg_file, 'w') as file_out:
        file_out.write(test_input)
    monkeypatch.setattr(checkbrltty, 'BrlttyConf', test_cfg_file)
    (migrate_file, migrate_bt, migrate_espeak,) = checkbrltty.check_for_unsupported_cfg()

    if expected_migrate_bt or expected_migrate_espeak:
        assert test_cfg_file == migrate_file
    assert expected_migrate_bt == migrate_bt
    assert expected_migrate_espeak == migrate_espeak



from leapp.actors import Actor
from leapp.libraries.actor.scansaphana import perform_sap_hana_scan
from leapp.models import SapHanaInfo
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class ScanSapHana(Actor):
    """
    Gathers information related to SAP HANA instances on the system.

    This actor collects information from SAP HANA installations and produces a message containing the details.
    The actor will determine whether SAP HANA is installed, running and which version is present on the system.
    """

    name = 'scan_sap_hana'
    consumes = ()
    produces = (SapHanaInfo,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        perform_sap_hana_scan()



import os
import shutil
import stat
import tempfile

from leapp.libraries.actor.spamassassinconfigupdate_backup import backup_file


def test_backup_file():
    tmpdir = tempfile.mkdtemp()
    try:
        file_path = os.path.join(tmpdir, 'foo-bar')
        content = 'test content\n'
        with open(file_path, 'w') as f:
            f.write(content)

        backup_path = backup_file(file_path)

        assert os.path.basename(backup_path) == 'foo-bar.leapp-backup'
        assert os.path.dirname(backup_path) == tmpdir
        assert len(os.listdir(tmpdir)) == 2
        st = os.stat(backup_path)
        assert stat.S_IMODE(st.st_mode) == (stat.S_IRUSR | stat.S_IWUSR)
        with open(backup_path, 'r') as f:
            backup_content = f.read()
        assert backup_content == content
        with open(file_path, 'r') as f:
            orig_content = f.read()
        assert orig_content == content
    finally:
        shutil.rmtree(tmpdir)


def test_backup_file_target_exists():
    tmpdir = tempfile.mkdtemp()
    try:
        file_path = os.path.join(tmpdir, 'foo-bar')
        primary_target_path = '%s.leapp-backup' % file_path
        primary_target_content = 'do not overwrite me'
        content = 'test_content\n'
        with open(file_path, 'w') as f:
            f.write(content)
        with open(primary_target_path, 'w') as f:
            f.write(primary_target_content)

        backup_path = backup_file(file_path)

        assert os.path.basename(backup_path).startswith('foo-bar.leapp-backup.')
        assert os.path.dirname(backup_path) == tmpdir
        assert len(os.listdir(tmpdir)) == 3
        st = os.stat(backup_path)
        assert stat.S_IMODE(st.st_mode) == (stat.S_IRUSR | stat.S_IWUSR)
        with open(backup_path, 'r') as f:
            assert f.read() == content
        with open(primary_target_path, 'r') as f:
            assert f.read() == primary_target_content
    finally:
        shutil.rmtree(tmpdir)



from leapp.models import fields, Model
from leapp.topics import BootPrepTopic


class BootContent(Model):
    """
    For information about what Leapp copies to the /boot/. We need to pass this information
    at least to the actors performing /boot/ cleanup.
    """
    topic = BootPrepTopic

    kernel_path = fields.String(help='Filepath of the kernel copied to /boot/ by Leapp.')
    initram_path = fields.String(help='Filepath of the initramfs copied to /boot/ by Leapp.')
    kernel_hmac_path = fields.String(help='Filepath of the kernel hmac copied to /boot/ by Leapp.')



from leapp.actors import Actor
from leapp.libraries.common import efi_reboot_fix
from leapp.tags import FinalizationPhaseTag, IPUWorkflowTag


class EfiFinalizationFix(Actor):
    """
    Adjust EFI boot entry for final reboot
    """

    name = 'efi_finalization_fix'
    consumes = ()
    produces = ()
    tags = (FinalizationPhaseTag, IPUWorkflowTag)

    def process(self):
        efi_reboot_fix.maybe_emit_updated_boot_entry()



import errno
import hashlib

from leapp.libraries.stdlib import api

VSFTPD_CONFIG_DIR = '/etc/vsftpd'
VSFTPD_DEFAULT_CONFIG_PATH = '/etc/vsftpd/vsftpd.conf'
STRICT_SSL_READ_EOF = 'strict_ssl_read_eof'
TCP_WRAPPERS = 'tcp_wrappers'


def read_file(path):
    """
    Read a file in text mode and return the contents.

    :param path: File path
    """
    with open(path, 'r') as f:
        return f.read()


def get_config_contents(path, read_func=read_file):
    """
    Try to read a vsftpd configuration file

    Try to read a vsftpd configuration file, log a warning if an error happens.
    :param path: File path
    :param read_func: Function to use to read the file. This is meant to be overridden in tests.
    :return: File contents or None, if the file could not be read
    """
    try:
        return read_func(path)
    except IOError as e:
        if e.errno != errno.ENOENT:
            api.current_logger().warning('Failed to read vsftpd configuration file: %s' % e)
        return None


def get_default_config_hash(read_func=read_file):
    """
    Read the default vsftpd configuration file (/etc/vsftpd/vsftpd.conf) and return its hash.

    :param read_func: Function to use to read the file. This is meant to be overridden in tests.
    :return SHA1 hash of the configuration file, or None if the file could not be read.
    """
    content = get_config_contents(VSFTPD_DEFAULT_CONFIG_PATH, read_func=read_func)
    if content is None:
        return None
    content = content.encode(encoding='utf-8')
    h = hashlib.sha1(content)
    return h.hexdigest()



import pytest

from leapp.libraries.actor.sanebackendsmigrate import NEW_QUIRKS, update_sane

testdata = [
    {'sane-backends': '/etc/sane.d/canon_dr.conf'},
    {'sane-backends': ''},
    {'ble': ''}
]


class MockLogger(object):
    def __init__(self):
        self.debugmsg = ''
        self.errmsg = ''

    def debug(self, message):
        self.debugmsg += message

    def error(self, message):
        self.errmsg += message


class MockPackage(object):
    def __init__(self, name, config):
        self.name = name
        self.config = config
        self.config_content = ''


class MockPackageSet(object):
    def __init__(self):
        self.installed_packages = None

    def add_packages(self, pkgs):
        if self.installed_packages is None:
            self.installed_packages = []

        for rpm, config in pkgs.items():
            self.installed_packages.append(MockPackage(rpm, config))

    def is_installed(self, pkg):
        for rpm in self.installed_packages:
            if pkg == rpm.name:
                return True
        return False

    def append_content(self, path, content):
        found = False

        for rpm in self.installed_packages:
            if path == rpm.config:
                found = True
                rpm.config_content += content
        if not found:
            raise IOError('Error during writing to file: {}.'.format(path))

    def check_content(self, path, content):
        found = False

        for rpm in self.installed_packages:
            if path == rpm.config and content in rpm.config_content:
                found = True

        return found


class ExpectedOutput(object):
    def __init__(self):
        self.debugmsg = ''
        self.errmsg = ''

    def create(self, rpms):
        error_list = []
        found = False

        for pkg, config in rpms.items():
            if pkg == 'sane-backends':
                found = True
                break

        if found:
            for sane_config in NEW_QUIRKS.keys():
                self.debugmsg += ('Updating SANE configuration file {}.'
                                  .format(sane_config))
                if config == '' or config != sane_config:
                    error_list.append((sane_config,
                                       'Error during writing to file: {}.'
                                       .format(sane_config)))

        if error_list:
            self.errmsg = ('The files below have not been modified '
                           '(error message included):' +
                           ''.join(['\n    - {}: {}'.format(err[0], err[1])
                                   for err in error_list]))


@pytest.mark.parametrize("rpms", testdata)
def test_actor_check_report(rpms):
    logger = MockLogger()
    installed_packages = MockPackageSet()

    installed_packages.add_packages(rpms)

    expected = ExpectedOutput()
    expected.create(rpms)

    update_sane(logger.debug,
                logger.error,
                installed_packages.is_installed,
                installed_packages.append_content,
                installed_packages.check_content)

    assert expected.debugmsg == logger.debugmsg
    assert expected.errmsg == logger.errmsg



class ParsingError(Exception):
    pass


class VsftpdConfigOptionParser(object):
    def _get_string_options(self):
        return ["secure_chroot_dir", "ftp_username", "chown_username", "xferlog_file",
                "vsftpd_log_file", "message_file", "nopriv_user", "ftpd_banner",
                "banned_email_file", "chroot_list_file", "pam_service_name", "guest_username",
                "userlist_file", "anon_root", "local_root", "banner_file", "pasv_address",
                "listen_address", "user_config_dir", "listen_address6", "cmds_allowed",
                "hide_file", "deny_file", "user_sub_token", "email_password_file",
                "rsa_cert_file", "dsa_cert_file", "dh_param_file", "ecdh_param_file",
                "ssl_ciphers", "rsa_private_key_file", "dsa_private_key_file", "ca_certs_file",
                "cmds_denied"]

    def _get_boolean_options(self):
        return ["anonymous_enable", "local_enable", "pasv_enable", "port_enable",
                "chroot_local_user", "write_enable", "anon_upload_enable",
                "anon_mkdir_write_enable", "anon_other_write_enable", "chown_uploads",
                "connect_from_port_20", "xferlog_enable", "dirmessage_enable",
                "anon_world_readable_only", "async_abor_enable", "ascii_upload_enable",
                "ascii_download_enable", "one_process_model", "xferlog_std_format",
                "pasv_promiscuous", "deny_email_enable", "chroot_list_enable",
                "setproctitle_enable", "text_userdb_names", "ls_recurse_enable",
                "log_ftp_protocol", "guest_enable", "userlist_enable", "userlist_deny",
                "use_localtime", "check_shell", "hide_ids", "listen", "port_promiscuous",
                "passwd_chroot_enable", "no_anon_password", "tcp_wrappers", "use_sendfile",
                "force_dot_files", "listen_ipv6", "dual_log_enable", "syslog_enable",
                "background", "virtual_use_local_privs", "session_support", "download_enable",
                "dirlist_enable", "chmod_enable", "secure_email_list_enable",
                "run_as_launching_user", "no_log_lock", "ssl_enable", "allow_anon_ssl",
                "force_local_logins_ssl", "force_local_data_ssl", "ssl_sslv2", "ssl_sslv3",
                "ssl_tlsv1", "ssl_tlsv1_1", "ssl_tlsv1_2", "tilde_user_enable",
                "force_anon_logins_ssl", "force_anon_data_ssl", "mdtm_write",
                "lock_upload_files", "pasv_addr_resolve", "reverse_lookup_enable",
                "userlist_log", "debug_ssl", "require_cert", "validate_cert",
                "strict_ssl_read_eof", "strict_ssl_write_shutdown", "ssl_request_cert",
                "delete_failed_uploads", "implicit_ssl", "ptrace_sandbox", "require_ssl_reuse",
                "isolate", "isolate_network", "ftp_enable", "http_enable", "seccomp_sandbox",
                "allow_writeable_chroot", "better_stou", "log_die"]

    def _get_integer_options(self):
        return ["accept_timeout", "connect_timeout", "local_umask", "anon_umask",
                "ftp_data_port", "idle_session_timeout", "data_connection_timeout",
                "pasv_min_port", "pasv_max_port", "anon_max_rate", "local_max_rate",
                "listen_port", "max_clients", "file_open_mode", "max_per_ip", "trans_chunk_size",
                "delay_failed_login", "delay_successful_login", "max_login_fails",
                "chown_upload_mode", "bind_retries"]

    def _get_boolean(self, option, value):
        value = value.upper()
        if value in ['YES', 'TRUE', '1']:
            return True
        if value in ['NO', 'FALSE', '0']:
            return False
        raise ParsingError("Boolean option '%s' contains a non-boolean value '%s'"
                           % (option, value))

    def _get_integer(self, option, value):
        try:
            return int(value)
        except ValueError:
            raise ParsingError("Integer option '%s' contains a non-integer value '%s'"
                               % (option, value))

    def parse_value(self, option, value):
        if option in self._get_string_options():
            return value
        if option in self._get_boolean_options():
            return self._get_boolean(option, value)
        if option in self._get_integer_options():
            return self._get_integer(option, value)

        raise ParsingError("Unknown option: '%s'" % option)


class VsftpdConfigParser(object):
    def __init__(self, config_content):
        self._option_parser = VsftpdConfigOptionParser()
        self.parsed_config = self._parse_config(config_content)

    def _parse_config_line(self, line, conf_dict):
        if not line or line.startswith('#') or line.isspace():
            return
        try:
            option, value = line.split('=', 1)
        except ValueError:
            raise ParsingError("The line does not have the form 'option=value': %s" % line)
        option = option.strip()
        value = value.strip()
        value = self._option_parser.parse_value(option, value)
        conf_dict[option] = value

    def _parse_config(self, contents):
        res = {}
        for (ix, line) in enumerate(contents.split('\n')):
            try:
                self._parse_config_line(line, res)
            except ParsingError as e:
                raise ParsingError("Syntax error on line %d: %s" % (ix + 1, e))
        return res



import pytest

from leapp.libraries.actor.cupsscanner import print_capabilities_check

testdata = (
    ('\n', False),
    ('Something else\n', False),
    ('#PrintcapFormat smth\n', False),
    ('PrintcapFormat\n', False),
    ('PrintcapFormat smth\n', True)
)


class MockCUPSD(object):
    def __init__(self, content):
        self.content = content

    def read(self, path):
        if path:
            return self.content.splitlines(True)
        return None


@pytest.mark.parametrize("content, expected", testdata)
def test_print_capabilities_check(content, expected):
    config = MockCUPSD(content)

    ret = print_capabilities_check('does_not_matter', config.read)

    assert ret == expected



import os

from leapp.libraries.stdlib import api

HOST_ROOT_MOUNT_BIND_PATH = '/installroot'
LOCAL_FILE_URL_PREFIX = 'file://'


def _adjust_local_file_url(repo_file_line):
    """
    Adjusts a local file url to the target user-space container in a provided
    repo file line by prefixing host root mount bind '/installroot' to it
    when needed.

    :param str repo_file_line: a line from a repo file
    :returns str: adjusted line or the provided line if no changes are needed
    """
    adjust_fields = ['baseurl', 'mirrorlist']

    if LOCAL_FILE_URL_PREFIX in repo_file_line and not repo_file_line.startswith('#'):
        entry_field, entry_value = repo_file_line.strip().split('=', 1)
        if not any(entry_field.startswith(field) for field in adjust_fields):
            return repo_file_line

        entry_value = entry_value.strip('\'\"')
        path = entry_value[len(LOCAL_FILE_URL_PREFIX):]
        new_entry_value = LOCAL_FILE_URL_PREFIX + os.path.join(HOST_ROOT_MOUNT_BIND_PATH, path.lstrip('/'))
        new_repo_file_line = entry_field + '=' + new_entry_value
        return new_repo_file_line
    return repo_file_line


def _extract_repos_from_repofile(context, repo_file):
    """
    Generator function that extracts repositories from a repo file in the given context
    and yields them as list of lines that belong to the repository.

    :param context: target user-space context
    :param str repo_file: path to repository file (inside the provided context)
    """
    with context.open(repo_file, 'r') as rf:
        repo_file_lines = rf.readlines()

    # Detect repo and remove lines before first repoid
    repo_found = False
    for idx, line in enumerate(repo_file_lines):
        if line.startswith('['):
            repo_file_lines = repo_file_lines[idx:]
            repo_found = True
            break

    if not repo_found:
        return

    current_repo = []
    for line in repo_file_lines:
        line = line.strip()

        if line.startswith('[') and current_repo:
            yield current_repo
            current_repo = []

        current_repo.append(line)
    yield current_repo


def _adjust_local_repos_to_container(context, repo_file, local_repoids):
    new_repo_file = []
    for repo in _extract_repos_from_repofile(context, repo_file):
        repoid = repo[0].strip('[]')
        adjusted_repo = repo
        if repoid in local_repoids:
            adjusted_repo = [_adjust_local_file_url(line) for line in repo]
        new_repo_file.append(adjusted_repo)

    # Combine the repo file contents into a string and write it back to the file
    new_repo_file = ['\n'.join(repo) for repo in new_repo_file]
    new_repo_file = '\n'.join(new_repo_file)
    with context.open(repo_file, 'w') as rf:
        rf.write(new_repo_file)


def process(context, target_repos_facts, iso_repoids, used_target_repoids):
    for repo_file_facts in target_repos_facts:
        repo_file_path = repo_file_facts.file
        local_repoids = set()
        for repo in repo_file_facts.data:
            # Skip repositories that aren't used or are provided by ISO
            if repo.repoid not in used_target_repoids or repo.repoid in iso_repoids:
                continue
            # Note repositories that contain local file url
            if repo.baseurl and LOCAL_FILE_URL_PREFIX in repo.baseurl or \
               repo.mirrorlist and LOCAL_FILE_URL_PREFIX in repo.mirrorlist:
                local_repoids.add(repo.repoid)

        if local_repoids:
            api.current_logger().debug(
                    'Adjusting following repos in the repo file - {}: {}'.format(repo_file_path,
                                                                                 ', '.join(local_repoids)))
            _adjust_local_repos_to_container(context, repo_file_path, local_repoids)



import json
import os

import pytest

from leapp.libraries.actor import persistentnetnamesconfig
from leapp.libraries.common.config import mock_configs
from leapp.libraries.common.testutils import CurrentActorMocked, logger_mocked, produce_mocked
from leapp.models import (
    InitrdIncludes,
    Interface,
    PCIAddress,
    PersistentNetNamesFacts,
    PersistentNetNamesFactsInitramfs,
    RenamedInterface,
    RenamedInterfaces,
    TargetInitramfsTasks
)

TEST_DIR = os.path.dirname(os.path.abspath(__file__))
CUR_DIR = ""


@pytest.fixture
def adjust_cwd():
    previous_cwd = os.getcwd()
    os.chdir(TEST_DIR)
    yield
    os.chdir(previous_cwd)


def generate_link_file_mocked(interface):
    return '/etc/systemd/network/10-leapp-{}.link'.format(interface.name)


def interface_mocked(i=0):
    return Interface(
        name='n{}'.format(i),
        devpath='dp{}'.format(i),
        driver='d{}'.format(i),
        vendor='v{}'.format(i),
        pci_info=PCIAddress(
            domain='pd{}'.format(i),
            bus='pb{}'.format(i),
            function='pf{}'.format(i),
            device='pd{}'.format(i)
        ),
        mac='m{}'.format(i)
    )


def generate_interfaces(count):
    return [interface_mocked(i) for i in range(count)]


def test_identical(current_actor_context):
    interfaces = generate_interfaces(4)
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interfaces))
    current_actor_context.feed(PersistentNetNamesFactsInitramfs(interfaces=interfaces))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    renamed_interfaces = current_actor_context.consume(RenamedInterfaces)[0]
    initrd_files = current_actor_context.consume(InitrdIncludes)[0]
    t_initrafms_tasks = current_actor_context.consume(TargetInitramfsTasks)[0]
    assert initrd_files.files == t_initrafms_tasks.include_files
    assert not renamed_interfaces.renamed
    assert not t_initrafms_tasks.include_files


def test_renamed_single_noneth(monkeypatch, current_actor_context):
    monkeypatch.setattr(persistentnetnamesconfig, 'generate_link_file', generate_link_file_mocked)

    interfaces = generate_interfaces(4)
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interfaces))
    interfaces[0].name = 'n4'
    current_actor_context.feed(PersistentNetNamesFactsInitramfs(interfaces=interfaces))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    renamed_interfaces = current_actor_context.consume(RenamedInterfaces)[0]
    initrd_files = current_actor_context.consume(InitrdIncludes)[0]
    t_initrafms_tasks = current_actor_context.consume(TargetInitramfsTasks)[0]
    assert initrd_files.files == t_initrafms_tasks.include_files
    assert not renamed_interfaces.renamed
    assert len(t_initrafms_tasks.include_files) == 1
    assert '/etc/systemd/network/10-leapp-n0.link' in t_initrafms_tasks.include_files


def test_renamed_swap_noneth(monkeypatch, current_actor_context):
    monkeypatch.setattr(persistentnetnamesconfig, 'generate_link_file', generate_link_file_mocked)

    interfaces = generate_interfaces(4)
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interfaces))
    interfaces[0].name = 'n3'
    interfaces[3].name = 'n0'
    current_actor_context.feed(PersistentNetNamesFactsInitramfs(interfaces=interfaces))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    renamed_interfaces = current_actor_context.consume(RenamedInterfaces)[0]
    initrd_files = current_actor_context.consume(InitrdIncludes)[0]
    t_initrafms_tasks = current_actor_context.consume(TargetInitramfsTasks)[0]
    assert initrd_files.files == t_initrafms_tasks.include_files
    assert not renamed_interfaces.renamed
    assert len(t_initrafms_tasks.include_files) == 2
    assert '/etc/systemd/network/10-leapp-n0.link' in t_initrafms_tasks.include_files
    assert '/etc/systemd/network/10-leapp-n3.link' in t_initrafms_tasks.include_files


def test_renamed_single_eth(monkeypatch, current_actor_context):
    monkeypatch.setattr(persistentnetnamesconfig, 'generate_link_file', generate_link_file_mocked)

    interfaces = generate_interfaces(4)
    for i in range(4):
        interfaces[i].name = 'eth{}'.format(i)
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interfaces))
    interfaces[0].name = 'eth4'
    current_actor_context.feed(PersistentNetNamesFactsInitramfs(interfaces=interfaces))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    renamed_interfaces = current_actor_context.consume(RenamedInterfaces)[0]
    initrd_files = current_actor_context.consume(InitrdIncludes)[0]
    t_initrafms_tasks = current_actor_context.consume(TargetInitramfsTasks)[0]
    assert initrd_files.files == t_initrafms_tasks.include_files
    assert len(renamed_interfaces.renamed) == 1
    assert renamed_interfaces.renamed[0].rhel7_name == 'eth0'
    assert renamed_interfaces.renamed[0].rhel8_name == 'eth4'
    assert not t_initrafms_tasks.include_files


def test_renamed_swap_eth(monkeypatch, current_actor_context):
    monkeypatch.setattr(persistentnetnamesconfig, 'generate_link_file', generate_link_file_mocked)

    interfaces = generate_interfaces(4)
    for i in range(4):
        interfaces[i].name = 'eth{}'.format(i)
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interfaces))
    interfaces[0].name = 'eth3'
    interfaces[3].name = 'eth0'
    current_actor_context.feed(PersistentNetNamesFactsInitramfs(interfaces=interfaces))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    renamed_interfaces = current_actor_context.consume(RenamedInterfaces)[0]
    initrd_files = current_actor_context.consume(InitrdIncludes)[0]
    t_initrafms_tasks = current_actor_context.consume(TargetInitramfsTasks)[0]
    assert initrd_files.files == t_initrafms_tasks.include_files
    assert len(renamed_interfaces.renamed) == 2
    for interface in renamed_interfaces.renamed:
        if interface.rhel7_name == 'eth0':
            assert interface.rhel8_name == 'eth3'
        elif interface.rhel7_name == 'eth3':
            assert interface.rhel8_name == 'eth0'
    assert not t_initrafms_tasks.include_files


def test_bz_1899455_crash_iface(monkeypatch, adjust_cwd):
    """
    Cover situation when network device is discovered on the src sys but not
    inside the upgrade environment.

    This typically happens when the network device needs specific drivers which
    are not present inside the upgrade initramfs. Usually it points to a missing
    actors that should influence the upgrade initramfs in a way the drivers are
    installed. In this situation, only correct thing we can do in this actor
    is print warning / report that we couldn't located particular devices so
    we cannot handle interface names related to this devices.
    """
    with open(os.path.join(CUR_DIR, 'files/crashed_ifaces.json')) as fp:
        json_msgs = json.load(fp)
    msgs = [
        PersistentNetNamesFacts.create(json_msgs["PersistentNetNamesFacts"]),
        PersistentNetNamesFactsInitramfs.create(json_msgs["PersistentNetNamesFactsInitramfs"]),
    ]
    monkeypatch.setattr(persistentnetnamesconfig, 'generate_link_file', generate_link_file_mocked)
    monkeypatch.setattr(persistentnetnamesconfig.api, 'current_actor', CurrentActorMocked(msgs=msgs))
    monkeypatch.setattr(persistentnetnamesconfig.api, 'current_logger', logger_mocked())
    monkeypatch.setattr(persistentnetnamesconfig.api, 'produce', produce_mocked())
    persistentnetnamesconfig.process()

    for prod_models in [RenamedInterfaces, InitrdIncludes, TargetInitramfsTasks]:
        any(isinstance(i, prod_models) for i in persistentnetnamesconfig.api.produce.model_instances)
    assert any(['Some network devices' in x for x in persistentnetnamesconfig.api.current_logger.warnmsg])


def test_no_network_renaming(monkeypatch):
    """
    This should cover OAMG-4243.
    """
    # this mock should be needed, as this function should be called, but just
    # for a check..
    monkeypatch.setattr(persistentnetnamesconfig, 'generate_link_file', generate_link_file_mocked)

    interfaces = generate_interfaces(4)
    for i in range(4):
        interfaces[i].name = 'myinterface{}'.format(i)
    msgs = [PersistentNetNamesFacts(interfaces=interfaces)]
    interfaces[0].name = 'changedinterfacename0'
    msgs.append(PersistentNetNamesFactsInitramfs(interfaces=interfaces))
    mocked_actor = CurrentActorMocked(msgs=msgs, envars={'LEAPP_NO_NETWORK_RENAMING': '1'})
    monkeypatch.setattr(persistentnetnamesconfig.api, 'current_actor', mocked_actor)
    monkeypatch.setattr(persistentnetnamesconfig.api, 'current_logger', logger_mocked())
    monkeypatch.setattr(persistentnetnamesconfig.api, 'produce', produce_mocked())
    persistentnetnamesconfig.process()

    ilog = 'Skipping handling of possibly renamed network interfaces: leapp executed with LEAPP_NO_NETWORK_RENAMING=1'
    assert ilog in persistentnetnamesconfig.api.current_logger.infomsg
    assert not persistentnetnamesconfig.api.produce.called



import os
from collections import defaultdict

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.config.version import get_source_major_version, get_target_major_version
from leapp.libraries.common.fetch import load_data_asset
from leapp.libraries.common.rpms import get_leapp_packages, LeappComponents
from leapp.libraries.stdlib import api
from leapp.models import PESIDRepositoryEntry, RepoMapEntry, RepositoriesMapping
from leapp.models.fields import ModelViolationError

OLD_REPOMAP_FILE = 'repomap.csv'
"""The name of the old, deprecated repository mapping file (no longer used)."""

REPOMAP_FILE = 'repomap.json'
"""The name of the new repository mapping file."""


class RepoMapData(object):
    VERSION_FORMAT = '1.2.0'

    def __init__(self):
        self.repositories = []
        self.mapping = {}

    def add_repository(self, data, pesid):
        """
        Add new PESIDRepositoryEntry with given pesid from the provided dictionary.

        :param data: A dict containing the data of the added repository. The dictionary structure corresponds
                     to the repositories entries in the repository mapping JSON schema.
        :type data: Dict[str, str]
        :param pesid: PES id of the repository family that the newly added repository belongs to.
        :type pesid: str
        """
        self.repositories.append(PESIDRepositoryEntry(
            repoid=data['repoid'],
            channel=data['channel'],
            rhui=data.get('rhui', ''),
            repo_type=data['repo_type'],
            arch=data['arch'],
            major_version=data['major_version'],
            pesid=pesid
        ))

    def get_repositories(self, valid_major_versions):
        """
        Return the list of PESIDRepositoryEntry object matching the specified major versions.
        """
        return [repo for repo in self.repositories if repo.major_version in valid_major_versions]

    def add_mapping(self, source_major_version, target_major_version, source_pesid, target_pesid):
        """
        Add a new mapping entry that is mapping the source pesid to the destination pesid(s),
        relevant in an IPU from the supplied source major version to the supplied target
        major version.

        :param str source_major_version: Specifies the major version of the source system
                                         for which the added mapping applies.
        :param str target_major_version: Specifies the major version of the target system
                                         for which the added mapping applies.
        :param str source_pesid: PESID of the source repository.
        :param Union[str|List[str]] target_pesid: A single target PESID or a list of target
                                                  PESIDs of the added mapping.
        """
        # NOTE: it could be more simple, but I prefer to be sure the input data
        # contains just one map per source PESID.
        key = '{}:{}'.format(source_major_version, target_major_version)
        rmap = self.mapping.get(key, defaultdict(set))
        self.mapping[key] = rmap
        if isinstance(target_pesid, list):
            rmap[source_pesid].update(target_pesid)
        else:
            rmap[source_pesid].add(target_pesid)

    def get_mappings(self, src_major_version, dst_major_version):
        """
        Return the list of RepoMapEntry objects for the specified upgrade path.

        IOW, the whole mapping for specified IPU.
        """
        key = '{}:{}'.format(src_major_version, dst_major_version)
        rmap = self.mapping.get(key, None)
        if not rmap:
            return None
        map_list = []
        for src_pesid in sorted(rmap.keys()):
            map_list.append(RepoMapEntry(source=src_pesid, target=sorted(rmap[src_pesid])))
        return map_list

    @staticmethod
    def load_from_dict(data):
        if data['version_format'] != RepoMapData.VERSION_FORMAT:
            raise ValueError(
                'The obtained repomap data has unsupported version of format.'
                ' Get {} required {}'
                .format(data['version_format'], RepoMapData.VERSION_FORMAT)
            )

        repomap = RepoMapData()

        # Load reposiories
        existing_pesids = set()
        for repo_family in data['repositories']:
            existing_pesids.add(repo_family['pesid'])
            for repo in repo_family['entries']:
                repomap.add_repository(repo, repo_family['pesid'])

        # Load mappings
        for mapping in data['mapping']:
            for entry in mapping['entries']:
                if not isinstance(entry['target'], list):
                    raise ValueError(
                        'The target field of a mapping entry is not a list: {}'
                        .format(entry)
                    )

                for pesid in [entry['source']] + entry['target']:
                    if pesid not in existing_pesids:
                        raise ValueError(
                            'The {} pesid is not related to any repository.'
                            .format(pesid)
                        )
                repomap.add_mapping(
                    source_major_version=mapping['source_major_version'],
                    target_major_version=mapping['target_major_version'],
                    source_pesid=entry['source'],
                    target_pesid=entry['target'],
                )
        return repomap


def _inhibit_upgrade(msg):
    local_path = os.path.join('/etc/leapp/file', REPOMAP_FILE)
    hint = (
        'All official data files are nowadays part of the installed rpms.'
        ' This issue is usually encountered when the data files are incorrectly customized, replaced, or removed'
        ' (e.g. by custom scripts).'
        ' In case you want to recover the original {lp} file, remove the current one (if it still exists)'
        ' and reinstall the following packages: {rpms}.'
        .format(
            lp=local_path,
            rpms=', '.join(get_leapp_packages(component=LeappComponents.REPOSITORY))
        )
    )
    raise StopActorExecutionError(msg, details={'hint': hint})


def _read_repofile(repofile):
    # NOTE(pstodulk): load_data_assert raises StopActorExecutionError, see
    # the code for more info. Keeping the handling on the framework in such
    # a case as we have no work to do in such a case here.
    repofile_data = load_data_asset(api.current_actor(),
                                    repofile,
                                    asset_fulltext_name='Repositories mapping',
                                    docs_url='',
                                    docs_title='')
    return repofile_data


def scan_repositories(read_repofile_func=_read_repofile):
    """
    Scan the repository mapping file and produce RepositoriesMap msg.

    See the description of the actor for more details.
    """
    # TODO: add filter based on the current arch
    # TODO: deprecate the product type and introduce the "channels" ?.. more or less
    # NOTE: product type is changed, now it's channel: eus,e4s,aus,tus,ga,beta

    if os.path.exists(os.path.join('/etc/leapp/files', OLD_REPOMAP_FILE)):
        # NOTE: what about creating the report (instead of warning)
        api.current_logger().warning(
            'The old repomap file /etc/leapp/files/repomap.csv is present.'
            ' The file has been replaced by the repomap.json file and it is'
            ' not used anymore.'
        )

    json_data = read_repofile_func(REPOMAP_FILE)
    try:
        repomap_data = RepoMapData.load_from_dict(json_data)
        mapping = repomap_data.get_mappings(get_source_major_version(), get_target_major_version())

        valid_major_versions = [get_source_major_version(), get_target_major_version()]
        api.produce(RepositoriesMapping(
            mapping=mapping,
            repositories=repomap_data.get_repositories(valid_major_versions)
        ))
    except ModelViolationError as err:
        err_message = (
            'The repository mapping file is invalid: '
            'the JSON does not match required schema (wrong field type/value): {}'
            .format(err)
        )
        _inhibit_upgrade(err_message)
    except KeyError as err:
        _inhibit_upgrade(
            'The repository mapping file is invalid: the JSON is missing a required field: {}'.format(err))
    except ValueError as err:
        # The error should contain enough information, so we do not need to clarify it further
        _inhibit_upgrade('The repository mapping file is invalid: {}'.format(err))



import os

from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.config import version
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import StorageInfo, TargetOSInstallationImage


def inhibit_if_not_valid_iso_file(iso):
    inhibit_title = None
    target_os = 'RHEL {}'.format(version.get_target_major_version())
    if not os.path.exists(iso.path):
        inhibit_title = 'Provided {target_os} installation ISO does not exists.'.format(target_os=target_os)
        inhibit_summary_tpl = 'The supplied {target_os} ISO path \'{iso_path}\' does not point to an existing file.'
        inhibit_summary = inhibit_summary_tpl.format(target_os=target_os, iso_path=iso.path)
    else:
        try:
            # TODO(mhecko): Figure out whether we will keep this since the scan actor is mounting the ISO anyway
            file_cmd_output = run(['file', '--mime', iso.path])
            if 'application/x-iso9660-image' not in file_cmd_output['stdout']:
                inhibit_title = 'Provided {target_os} installation image is not a valid ISO.'.format(
                        target_os=target_os)
                summary_tpl = ('The provided {target_os} installation image path \'{iso_path}\''
                               'does not point to a valid ISO image.')
                inhibit_summary = summary_tpl.format(target_os=target_os, iso_path=iso.path)

        except CalledProcessError as err:
            raise StopActorExecutionError(message='Failed to check whether {0} is an ISO file.'.format(iso.path),
                                          details={'details': '{}'.format(err)})
    if inhibit_title:
        remediation_hint = ('Check whether the supplied target OS installation path points to a valid'
                            '{target_os} ISO image.'.format(target_os=target_os))

        reporting.create_report([
            reporting.Title(inhibit_title),
            reporting.Summary(inhibit_summary),
            reporting.Remediation(hint=remediation_hint),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.REPOSITORY]),
        ])
        return True
    return False


def inhibit_if_failed_to_mount_iso(iso):
    if iso.was_mounted_successfully:
        return False

    target_os = 'RHEL {0}'.format(version.get_target_major_version())
    title = 'Failed to mount the provided {target_os} installation image.'
    summary = 'The provided {target_os} installation image {iso_path} could not be mounted.'
    hint = 'Verify that the provided ISO is a valid {target_os} installation image'
    reporting.create_report([
        reporting.Title(title.format(target_os=target_os)),
        reporting.Summary(summary.format(target_os=target_os, iso_path=iso.path)),
        reporting.Remediation(hint=hint.format(target_os=target_os)),
        reporting.Severity(reporting.Severity.MEDIUM),
        reporting.Groups([reporting.Groups.INHIBITOR]),
        reporting.Groups([reporting.Groups.REPOSITORY]),
    ])
    return True


def inhibit_if_wrong_iso_rhel_version(iso):
    # If the major version could not be determined, the iso.rhel_version will be an empty string
    if not iso.rhel_version:
        reporting.create_report([
            reporting.Title(
                'Failed to determine RHEL version provided by the supplied installation image.'),
            reporting.Summary(
                'Could not determine what RHEL version does the supplied installation image'
                ' located at {iso_path} provide.'.format(iso_path=iso.path)
            ),
            reporting.Remediation(hint='Check that the supplied image is a valid RHEL installation image.'),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.REPOSITORY]),
        ])
        return

    iso_rhel_major_version = iso.rhel_version.split('.')[0]
    req_major_ver = version.get_target_major_version()
    if iso_rhel_major_version != req_major_ver:
        summary = ('The provided RHEL installation image provides RHEL {iso_rhel_ver}, however, a RHEL '
                   '{required_rhel_ver} image is required for the upgrade.')

        reporting.create_report([
            reporting.Title('The provided installation image provides invalid RHEL version.'),
            reporting.Summary(summary.format(iso_rhel_ver=iso.rhel_version, required_rhel_ver=req_major_ver)),
            reporting.Remediation(hint='Check that the supplied image is a valid RHEL installation image.'),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.REPOSITORY]),
        ])


def inhibit_if_iso_not_located_on_persistent_partition(iso):
    # Check whether the filesystem that on which the ISO resides is mounted in a persistent fashion
    storage_info = next(api.consume(StorageInfo), None)
    if not storage_info:
        raise StopActorExecutionError('Actor did not receive any StorageInfo message.')

    # Assumes that the path has been already checked for validity, e.g., the ISO path points to a file
    iso_mountpoint = iso.path
    while not os.path.ismount(iso_mountpoint):  # Guaranteed to terminate because we must reach / eventually
        iso_mountpoint = os.path.dirname(iso_mountpoint)

    is_iso_on_persistent_partition = False
    for fstab_entry in storage_info.fstab:
        if fstab_entry.fs_file == iso_mountpoint:
            is_iso_on_persistent_partition = True
            break

    if not is_iso_on_persistent_partition:
        target_ver = version.get_target_major_version()
        title = 'The RHEL {target_ver} installation image is not located on a persistently mounted partition'
        summary = ('The provided RHEL {target_ver} installation image {iso_path} is located'
                   ' on a partition without an entry in /etc/fstab, causing the partition '
                   ' to be persistently mounted.')
        hint = ('Move the installation image to a partition that is persistently mounted, or create an /etc/fstab'
                ' entry for the partition on which the installation image is located.')

        reporting.create_report([
            reporting.Title(title.format(target_ver=target_ver)),
            reporting.Summary(summary.format(target_ver=target_ver, iso_path=iso.path)),
            reporting.Remediation(hint=hint),
            reporting.RelatedResource('file', '/etc/fstab'),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.REPOSITORY]),
        ])


def inihibit_if_iso_does_not_contain_basic_repositories(iso):
    missing_basic_repoids = {'BaseOS', 'AppStream'}

    for custom_repo in iso.repositories:
        missing_basic_repoids.remove(custom_repo.repoid)
        if not missing_basic_repoids:
            break

    if missing_basic_repoids:
        target_ver = version.get_target_major_version()

        title = 'Provided RHEL {target_ver} installation ISO is missing fundamental repositories.'
        summary = ('The supplied RHEL {target_ver} installation ISO {iso_path} does not contain '
                   '{missing_repos} repositor{suffix}')
        hint = 'Check whether the supplied ISO is a valid RHEL {target_ver} installation image.'

        reporting.create_report([
            reporting.Title(title.format(target_ver=target_ver)),
            reporting.Summary(summary.format(target_ver=target_ver,
                                             iso_path=iso.path,
                                             missing_repos=','.join(missing_basic_repoids),
                                             suffix=('y' if len(missing_basic_repoids) == 1 else 'ies'))),
            reporting.Remediation(hint=hint.format(target_ver=target_ver)),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.REPOSITORY]),
        ])


def perform_target_iso_checks():
    requested_target_iso_msg_iter = api.consume(TargetOSInstallationImage)
    target_iso = next(requested_target_iso_msg_iter, None)

    if not target_iso:
        return

    if next(requested_target_iso_msg_iter, None):
        api.current_logger().warning('Received multiple msgs with target ISO to use.')

    # Cascade the inhibiting conditions so that we do not spam the user with inhibitors
    is_iso_invalid = inhibit_if_not_valid_iso_file(target_iso)
    if not is_iso_invalid:
        failed_to_mount_iso = inhibit_if_failed_to_mount_iso(target_iso)
        if not failed_to_mount_iso:
            inhibit_if_wrong_iso_rhel_version(target_iso)
            inhibit_if_iso_not_located_on_persistent_partition(target_iso)
            inihibit_if_iso_does_not_contain_basic_repositories(target_iso)



from leapp.libraries.actor.checkhacluster import CIB_LOCATION, COROSYNC_CONF_LOCATION
from leapp.reporting import Report
from leapp.utils.report import is_inhibitor


def assert_inhibits(reports, node_type):
    assert len(reports) == 1
    report_fields = reports[0].report
    assert is_inhibitor(report_fields)
    assert "cluster {0}".format(node_type) in report_fields["summary"]


def test_no_inhibit_when_no_ha_cluster(monkeypatch, current_actor_context):
    monkeypatch.setattr("os.path.isfile", lambda path: False)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_inhibits_when_cluster_node(monkeypatch, current_actor_context):
    # NOTE(ivasilev) Limiting here the paths to mock not to cause unexpected side-effects
    # (original test had path: True)
    monkeypatch.setattr("os.path.isfile", lambda path: path in (COROSYNC_CONF_LOCATION, CIB_LOCATION))
    current_actor_context.run()
    assert_inhibits(current_actor_context.consume(Report), "node")


def test_inhibits_when_cluster_node_no_cib(monkeypatch, current_actor_context):
    monkeypatch.setattr(
        "os.path.isfile",
        lambda path: path == COROSYNC_CONF_LOCATION
    )
    current_actor_context.run()
    assert_inhibits(current_actor_context.consume(Report), "node")


def test_inhibits_when_cluster_remote_node(monkeypatch, current_actor_context):
    # NOTE(ivasilev) Limiting here the paths to mock not to cause unexpected side-effects
    # (original test had path: path != COROSYNC_CONF_LOCATION)
    monkeypatch.setattr(
        "os.path.isfile",
        lambda path: path == CIB_LOCATION
    )
    current_actor_context.run()
    assert_inhibits(current_actor_context.consume(Report), "remote node")



import errno
from os import listdir, path

from leapp.libraries.stdlib import api
from leapp.models import IfCfg, IfCfgProperty

SYSCONFIG_DIR = "/etc/sysconfig/network-scripts"


def aux_file(prefix, filename):
    directory = path.dirname(filename)
    keys_base = path.basename(filename).replace("ifcfg-", prefix)
    return path.join(directory, keys_base)


def process_ifcfg(filename, secrets=False):
    if not path.exists(filename):
        return None

    properties = []
    for line in open(filename).readlines():
        try:
            (name, value) = line.split("#")[0].strip().split("=")
            if secrets:
                value = None
        except ValueError:
            # We're not interested in lines that are not
            # simple assignments. Play it safe.
            continue

        # Deal with simple quoting. We don't expand anything, nor do
        # multiline strings or anything of that sort.
        if value is not None and len(value) > 1 and value[0] == value[-1]:
            if value.startswith('"') or value.startswith("'"):
                value = value[1:-1]

        properties.append(IfCfgProperty(name=name, value=value))
    return properties


def process_plain(filename):
    if not path.exists(filename):
        return None
    return open(filename).readlines()


def process_file(filename):
    api.produce(IfCfg(
        filename=filename,
        properties=process_ifcfg(filename),
        secrets=process_ifcfg(aux_file("keys-", filename), secrets=True),
        rules=process_plain(aux_file("rule-", filename)),
        rules6=process_plain(aux_file("rule6-", filename)),
        routes=process_plain(aux_file("route-", filename)),
        routes6=process_plain(aux_file("route6-", filename)),
    ))


def process_dir(directory):
    try:
        keyfiles = listdir(directory)
    except OSError as e:
        if e.errno == errno.ENOENT:
            return
        raise

    for f in keyfiles:
        if f.startswith("ifcfg-"):
            process_file(path.join(directory, f))


def process():
    process_dir(SYSCONFIG_DIR)



from leapp.actors import Actor
from leapp.libraries.actor.checkhybridimage import check_hybrid_image
from leapp.models import FirmwareFacts, HybridImage, InstalledRPM
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckHybridImage(Actor):
    """
    Check if the system is using Azure hybrid image.

    These images have a default relative symlink to EFI
    partition even when booted using BIOS and in such cases
    GRUB is not able find "grubenv" to get the kernel cmdline
    options and fails to boot after upgrade`.
    """

    name = 'checkhybridimage'
    consumes = (InstalledRPM, FirmwareFacts)
    produces = (HybridImage, Report)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        check_hybrid_image()



import os

from leapp.libraries.actor import systemfacts
from leapp.models import DefaultGrub


class RunMocked(object):
    def __init__(self, cmd_result):
        self.called = 0
        self.cmd_result = cmd_result
        self.split = False
        self.cmd = None

    def __call__(self, cmd, split=False):
        self.cmd = cmd
        self.split = split
        self.called += 1
        return self.cmd_result


def test_default_grub_info_valid(monkeypatch):
    mocked_run = RunMocked({
        'stdout': [
            'line="whatever else here"',
            'newline="whatever"',
            '# comment here',
            'why_not=value',
            ' # whitespaces around comment ',
            ' ',
            ' last=last really'
        ],
    })
    expected_result = [
        DefaultGrub(name='line', value='"whatever else here"'),
        DefaultGrub(name='newline', value='"whatever"'),
        DefaultGrub(name='why_not', value='value'),
        DefaultGrub(name='last', value='last really'),
    ]
    monkeypatch.setattr(systemfacts, 'run', mocked_run)
    monkeypatch.setattr(os.path, 'isfile', lambda dummy: True)
    for msg in systemfacts._default_grub_info():
        expected_msg = expected_result.pop(0)
        assert msg.name == expected_msg.name
        assert msg.value == expected_msg.value
    assert mocked_run.called
    assert not expected_result



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic, SystemInfoTopic


class Authselect(Model):
    """
    Suggested changes that will convert the system to authselect.

    This model describes the authselect call that can be used to convert
    existing configuration into a equivalent or similar configuration
    that is generated by authselect.
    """
    topic = SystemFactsTopic

    profile = fields.Nullable(fields.String(default=None))
    """
    Suggested authselect profile name.
    """

    features = fields.List(fields.String())
    """
    Suggested authselect profile features.
    """

    confirm = fields.Boolean(default=True)
    """
    Changes to the system requires admin confirmation.
    """


class AuthselectDecision(Model):
    """
    Confirmation of changes suggested in Authselect model.

    If confirmed is True, the changes will be applied on RHEL-8 machine.
    """
    topic = SystemInfoTopic

    confirmed = fields.Boolean(default=False)
    """
    If true, authselect should be called after upgrade.
    """



import os

import leapp.libraries.common.config as ipu_config
from leapp.libraries.common.mounting import LoopMount, MountError
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import CustomTargetRepository, TargetOSInstallationImage


def determine_rhel_version_from_iso_mountpoint(iso_mountpoint):
    baseos_packages = os.path.join(iso_mountpoint, 'BaseOS/Packages')
    if os.path.isdir(baseos_packages):
        def is_rh_release_pkg(pkg_name):
            return pkg_name.startswith('redhat-release') and 'eula' not in pkg_name

        redhat_release_pkgs = [pkg for pkg in os.listdir(baseos_packages) if is_rh_release_pkg(pkg)]

        if not redhat_release_pkgs:
            return ''  # We did not determine anything

        if len(redhat_release_pkgs) > 1:
            api.current_logger().warning('Multiple packages with name redhat-release* found when '
                                         'determining RHEL version of the supplied installation ISO.')

        redhat_release_pkg = redhat_release_pkgs[0]

        determined_rhel_ver = ''
        try:
            rh_release_pkg_path = os.path.join(baseos_packages, redhat_release_pkg)
            # rpm2cpio is provided by rpm; cpio is a dependency of yum (rhel7) and a dependency of dracut which is
            # a dependency for leapp (rhel8+)
            cpio_archive = run(['rpm2cpio', rh_release_pkg_path])
            etc_rh_release_contents = run(['cpio', '--extract', '--to-stdout', './etc/redhat-release'],
                                          stdin=cpio_archive['stdout'])

            # 'Red Hat Enterprise Linux Server release 7.9 (Maipo)' -> ['Red Hat...', '7.9 (Maipo']
            product_release_fragments = etc_rh_release_contents['stdout'].split('release')
            if len(product_release_fragments) != 2:
                return ''  # Unlikely. Either way we failed to parse the release

            if not product_release_fragments[0].startswith('Red Hat'):
                return ''

            determined_rhel_ver = product_release_fragments[1].strip().split(' ', 1)[0]  # Remove release name (Maipo)
            return determined_rhel_ver
        except CalledProcessError:
            return ''
    return ''


def inform_ipu_about_request_to_use_target_iso():
    target_iso_path = ipu_config.get_env('LEAPP_TARGET_ISO')
    if not target_iso_path:
        return

    iso_mountpoint = '/iso'

    if not os.path.exists(target_iso_path):
        # If the path does not exists, do not attempt to mount it and let the upgrade be inhibited by the check actor
        api.produce(TargetOSInstallationImage(path=target_iso_path,
                                              repositories=[],
                                              mountpoint=iso_mountpoint,
                                              was_mounted_successfully=False))
        return

    # Mount the given ISO, extract the available repositories and determine provided RHEL version
    iso_scan_mountpoint = '/var/lib/leapp/iso_scan_mountpoint'
    try:
        with LoopMount(source=target_iso_path, target=iso_scan_mountpoint):
            required_repositories = ('BaseOS', 'AppStream')

            # Check what required repositories are present in the root of the ISO
            iso_contents = os.listdir(iso_scan_mountpoint)
            present_repositories = [req_repo for req_repo in required_repositories if req_repo in iso_contents]

            # Create custom repository information about the repositories found in the root of the ISO
            iso_repos = []
            for repo_dir in present_repositories:
                baseurl = 'file://' + os.path.join(iso_mountpoint, repo_dir)
                iso_repo = CustomTargetRepository(name=repo_dir, baseurl=baseurl, repoid=repo_dir)
                api.produce(iso_repo)
                iso_repos.append(iso_repo)

            rhel_version = determine_rhel_version_from_iso_mountpoint(iso_scan_mountpoint)

            api.produce(TargetOSInstallationImage(path=target_iso_path,
                                                  repositories=iso_repos,
                                                  mountpoint=iso_mountpoint,
                                                  rhel_version=rhel_version,
                                                  was_mounted_successfully=True))
    except MountError:
        # Do not analyze the situation any further as ISO checks will be done by another actor
        iso_mountpoint = '/iso'
        api.produce(TargetOSInstallationImage(path=target_iso_path,
                                              repositories=[],
                                              mountpoint=iso_mountpoint,
                                              was_mounted_successfully=False))



from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM


def get_kernel_rpm_release(rpm):
    """
    Get the release of a kernel RPM as an integer.

    :param rpm: An instance of an RPM derived model.
    """
    return int(rpm.release.split('.')[0])


def get_kernel_debug_rpms():
    """
    Get all installed kernel-debug packages ordered by release number (ascending).
    """
    rpms = next(api.consume(DistributionSignedRPM), DistributionSignedRPM())
    return sorted([pkg for pkg in rpms.items if pkg.name == 'kernel-debug'], key=get_kernel_rpm_release)


def process():
    pkgs = get_kernel_debug_rpms()
    if len(pkgs) > 1:
        title = 'Multiple debug kernels installed'
        summary = ('DNF cannot produce a valid upgrade transaction when'
                   ' multiple kernel-debug packages are installed.')
        hint = 'Remove all but one kernel-debug packages before running Leapp again.'
        all_but_latest_kernel_debug = pkgs[:-1]
        packages = ['{n}-{v}-{r}'.format(n=pkg.name, v=pkg.version, r=pkg.release)
                    for pkg in all_but_latest_kernel_debug]
        commands = [['yum', '-y', 'remove'] + packages]
        reporting.create_report([
            reporting.Title(title),
            reporting.Summary(summary),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.KERNEL]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Remediation(hint=hint, commands=commands),
            reporting.RelatedResource('package', 'kernel-debug')
        ])



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class DNFWorkaround(Model):
    """
    DNFWorkaround is used to register scripts, that have to be executed to apply modifications to the system,
    just before DNF performs a transaction in order for it to succeed.

    As an example in the case of the RHEL7 to RHEL8 upgrade, we have to execute a script to fixup the way how
    yum and dnf symlinks are created as they cannot be replaced by RPM. To solve this we have created the
    handleyumconfig tool in the system_upgrade/el7toel8 repository and register the workaround with the
    registeryumadjustment actor.
    """
    topic = SystemInfoTopic

    script_path = fields.String()
    """
    Absolute path to a bash script to execute
    """

    script_args = fields.List(fields.String(), default=[])
    """
    Arguments with which the script should be executed

    In case that an argument contains a whitespace or an escapable character,
    the argument must be already treated correctly. e.g.
        `script_args = ['-i', 'my\\ string']
    """

    display_name = fields.String()
    """
    Name to display for this script when executed
    """



import os

from leapp.libraries.stdlib import api, CalledProcessError, run

DEFAULT_OPENSSL_CONF = '/etc/pki/tls/openssl.cnf'
OPENSSL_CONF_RPMNEW = '{}.rpmnew'.format(DEFAULT_OPENSSL_CONF)
OPENSSL_CONF_BACKUP = '{}.leappsave'.format(DEFAULT_OPENSSL_CONF)


def _is_openssl_modified():
    """
    Return True if modified in any way
    """
    # NOTE(pstodulk): this is different from the approach in scansourcefiles,
    # where we are interested about modified content. In this case, if the
    # file is modified in any way, let's do something about that..
    try:
        run(['rpm', '-Vf', DEFAULT_OPENSSL_CONF])
    except CalledProcessError:
        return True
    return False


def _safe_mv_file(src, dst):
    """
    Move the file from src to dst. Return True on success, otherwise False.
    """
    try:
        run(['mv', src, dst])
    except CalledProcessError:
        return False
    return True


def process():
    if not _is_openssl_modified():
        return
    if not os.path.exists(OPENSSL_CONF_RPMNEW):
        api.current_logger().debug('The {} file is modified, but *.rpmsave not found. Cannot do anything.')
        return
    if not _safe_mv_file(DEFAULT_OPENSSL_CONF, OPENSSL_CONF_BACKUP):
        # NOTE(pstodulk): One of reasons could be the file is missing, however
        # that's not expected to happen at all. If the file is missing before
        # the upgrade, it will be installed by new openssl* package
        api.current_logger().error(
            'Could not back up the {} file. Skipping other actions.'
            .format(DEFAULT_OPENSSL_CONF)
        )
        return
    if not _safe_mv_file(OPENSSL_CONF_RPMNEW, DEFAULT_OPENSSL_CONF):
        # unexpected, it's double seatbelt
        api.current_logger().error('Cannot apply the new openssl configuration file! Restore it from the backup.')
        if not _safe_mv_file(OPENSSL_CONF_BACKUP, DEFAULT_OPENSSL_CONF):
            api.current_logger().error('Cannot restore the openssl configuration file!')



import errno

from leapp.libraries.actor import spamassassinconfigupdate_spamc
from leapp.libraries.common.spamassassinutils import SPAMC_CONFIG_FILE
from leapp.libraries.common.testutils import make_IOError, make_OSError
from leapp.models.spamassassinfacts import SpamassassinFacts

# The test cases reuse values from the SpamassassinConfigRead test cases


def test_rewrite_spamc_config():
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl sslv3\n')
    assert new_content == '--ssl\n'

    # equal sign format
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl=tlsv1\n')
    assert new_content == '--ssl\n'

    # no argument
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl\n')
    assert new_content == '--ssl\n'


def test_rewrite_spamc_config_without_valid_argument():
    # If we encounter an unrecognized parameter, we leave it be - the
    # configuration is invalid anyway, so let's not mess it up even more.
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl foo\n')
    assert new_content == '--ssl foo\n'

    # --ssl followed by another option
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl -B\n')
    assert new_content == '--ssl -B\n'

    # space surrounding the equal sign. This amounts to an unrecognized
    # argument (empty string)
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl= tlsv1\n')
    assert new_content == '--ssl= tlsv1\n'

    # space surrounding the equal sign. This amounts to an unrecognized
    # argument ("=tlsv1")
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl =tlsv1\n')
    assert new_content == '--ssl =tlsv1\n'


def test_rewrite_spamc_config_multiline():
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('-B --ssl \n sslv3 -c\n-H\n')
    assert new_content == '-B --ssl \n -c\n-H\n'

    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl\n-B\n')
    assert new_content == '--ssl\n-B\n'


def test_rewrite_spamc_config_tls_supersedes_ssl():
    # Ideally, the result would be a single '--ssl' option. However for
    # simplicity, we allow the option to be output twice. It's not nice, but
    # there's nothing technically wrong with it. And it's really a corner case
    # anyway.  If someone fixes it, this test case can be updated to expect
    # '--ssl' as output.
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl sslv3 --ssl tlsv1\n')
    assert new_content == '--ssl --ssl\n'

    # reverse order
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl tlsv1 --ssl sslv3\n')
    assert new_content == '--ssl --ssl\n'


def test_rewrite_spamc_config_comment():
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('# --ssl tlsv1\n')
    assert new_content == '# --ssl tlsv1\n'

    # comment mixed with actual option
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('# --ssl tlsv1\n--ssl sslv3\n')
    assert new_content == '# --ssl tlsv1\n--ssl\n'

    # comment mixed with actual option
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl tlsv1\n# --ssl sslv3\n')
    assert new_content == '--ssl\n# --ssl sslv3\n'


def test_rewrite_spamc_config_crazy_corner_cases():
    # The option and new_content can have a comment in between
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl\n# foo\ntlsv1\n')
    assert new_content == '--ssl\n# foo\n\n'

    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl\n# foo\n# bar\nsslv3\n')
    assert new_content == '--ssl\n# foo\n# bar\n\n'

    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl\n# foo\n# tlsv1\nsslv3\n')
    assert new_content == '--ssl\n# foo\n# tlsv1\n\n'

    # --ssl followed by another option
    new_content = spamassassinconfigupdate_spamc._rewrite_spamc_config('--ssl\n# foo\n-B\n')
    assert new_content == '--ssl\n# foo\n-B\n'


class MockBackup(object):
    def __init__(self, to_raise=None):
        self.to_raise = to_raise
        self.called = 0
        self.paths = []

    def __call__(self, path):
        self.called += 1
        self.paths.append(path)
        if self.to_raise:
            raise self.to_raise
        return '/path/to/backup'


class MockFileOperations(object):
    def __init__(self, read_error=None, write_error=None):
        self.files = {}
        self.files_read = {}
        self.files_written = {}
        self.read_called = 0
        self.write_called = 0
        self.read_error = read_error
        self.write_error = write_error

    def _increment_read_counters(self, path):
        self.read_called += 1
        self.files_read.setdefault(path, 0)
        self.files_read[path] += 1

    def read(self, path):
        self._increment_read_counters(path)
        if self.read_error:
            raise self.read_error
        return self.files[path]

    def _increment_write_counters(self, path):
        self.write_called += 1
        self.files_written.setdefault(path, 0)
        self.files_written[path] += 1

    def write(self, path, content):
        self._increment_write_counters(path)
        if self.write_error:
            raise self.write_error
        self.files[path] = content


def test_migrate_spamc_config():
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    fileops = MockFileOperations()
    fileops.files[SPAMC_CONFIG_FILE] = '--ssl sslv3\n# foo\n-B\n'
    backup_func = MockBackup()

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    assert backup_func.called == 1
    assert backup_func.paths[0] == SPAMC_CONFIG_FILE
    assert fileops.files[SPAMC_CONFIG_FILE] == '--ssl\n# foo\n-B\n'
    assert fileops.read_called == 1
    assert fileops.files_read[SPAMC_CONFIG_FILE] == 1
    assert fileops.write_called == 1
    assert fileops.files_written[SPAMC_CONFIG_FILE] == 1


def test_migrate_spamc_config_no_ssl_option():
    facts = SpamassassinFacts(spamc_ssl_argument=None, service_overriden=False)
    fileops = MockFileOperations()
    backup_func = MockBackup()

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    assert backup_func.called == 0
    assert fileops.read_called == 0
    assert fileops.write_called == 0


def test_migrate_spamc_config_no_write_if_backup_fails():
    # OSError (e.g. os.open)
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    fileops = MockFileOperations()
    backup_func = MockBackup(to_raise=make_OSError(errno.EACCES))

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    assert fileops.write_called == 0

    # IOError (e.g. file.read)
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    fileops = MockFileOperations()
    backup_func = MockBackup(to_raise=make_IOError(errno.EACCES))

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    assert fileops.write_called == 0


def test_migrate_spamc_config_read_failure():
    # OSError (e.g. os.open)
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    fileops = MockFileOperations(read_error=make_OSError(errno.EACCES))
    backup_func = MockBackup()

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    # The main purpose of this test is to check that exceptions are handled
    # properly. The following assertions are supplementary.
    assert fileops.read_called == 1
    assert fileops.write_called == 0

    # IOError (e.g. builtin open)
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    fileops = MockFileOperations(read_error=make_IOError(errno.EACCES))
    backup_func = MockBackup()

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    assert fileops.read_called == 1
    assert fileops.write_called == 0


def test_migrate_spamc_config_write_failure():
    # OSError (e.g. os.open)
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    fileops = MockFileOperations(write_error=make_OSError(errno.EACCES))
    fileops.files[SPAMC_CONFIG_FILE] = '--ssl sslv3\n# foo\n-B\n'
    backup_func = MockBackup()

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    # The main purpose of this test is to check that exceptions are handled
    # properly. The following assertions are supplementary.
    assert fileops.read_called == 1
    assert fileops.write_called == 1

    # IOError (e.g. builtin open)
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    fileops = MockFileOperations(write_error=make_IOError(errno.EACCES))
    fileops.files[SPAMC_CONFIG_FILE] = '--ssl sslv3\n# foo\n-B\n'
    backup_func = MockBackup()

    spamassassinconfigupdate_spamc.migrate_spamc_config(facts, fileops, backup_func)

    assert fileops.read_called == 1
    assert fileops.write_called == 1



import re
import shutil

BackupSuffix = '.bak'


def migrate_file(fn, migrate_bt, migrate_espeak):
    # nothing to migrate
    if not fn or (not migrate_bt and not migrate_espeak):
        return

    # make backup
    shutil.copy2(fn, fn + BackupSuffix)

    regex_bt = re.compile(r'\b(?:(?:bth)|(?:bluez))((?:[:\-][0-9a-fA-F]{2}){6})\b')
    regex_espeak = re.compile(r'^(\s*speech-driver\s+)es\b')

    with open(fn, 'w') as file_out:
        with open(fn + BackupSuffix) as file_in:
            for line in file_in:
                if migrate_bt and regex_bt.search(line):
                    line = regex_bt.sub(r'bluetooth\1', line)
                elif migrate_espeak and regex_espeak.search(line):
                    line = regex_espeak.sub(r'\1en', line)
                file_out.write(line)



import os
import sys
import uuid

from leapp.cli.commands import command_utils
from leapp.cli.commands.config import get_config
from leapp.cli.commands.upgrade import breadcrumbs, util
from leapp.exceptions import CommandError, LeappError
from leapp.logger import configure_logger
from leapp.utils.audit import Execution
from leapp.utils.clicmd import command, command_opt
from leapp.utils.output import beautify_actor_exception, report_errors, report_info


@command('preupgrade', help='Generate preupgrade report')
@command_opt('whitelist-experimental', action='append', metavar='ActorName', help='Enables experimental actors')
@command_opt('debug', is_flag=True, help='Enable debug mode', inherit=False)
@command_opt('verbose', is_flag=True, help='Enable verbose logging', inherit=False)
@command_opt('no-rhsm', is_flag=True, help='Use only custom repositories and skip actions'
                                           ' with Red Hat Subscription Manager')
@command_opt('no-insights-register', is_flag=True, help='Do not register into Red Hat Insights')
@command_opt('no-rhsm-facts', is_flag=True, help='Do not store migration information using Red Hat '
                                                 'Subscription Manager. Automatically implied by --no-rhsm.')
@command_opt('enablerepo', action='append', metavar='<repoid>',
             help='Enable specified repository. Can be used multiple times.')
@command_opt('channel',
             help='Set preferred channel for the IPU target.',
             choices=['ga', 'e4s', 'eus', 'aus'],
             value_type=str.lower)  # This allows the choices to be case insensitive
@command_opt('iso', help='Use provided target RHEL installation image to perform the in-place upgrade.')
@command_opt('target', choices=command_utils.get_supported_target_versions(),
             help='Specify RHEL version to upgrade to for {} detected upgrade flavour'.format(
                 command_utils.get_upgrade_flavour()))
@command_opt('report-schema', help='Specify report schema version for leapp-report.json',
             choices=['1.0.0', '1.1.0', '1.2.0'], default=get_config().get('report', 'schema'))
@command_opt('nogpgcheck', is_flag=True, help='Disable RPM GPG checks. Same as yum/dnf --nogpgcheck option.')
@breadcrumbs.produces_breadcrumbs
def preupgrade(args, breadcrumbs):
    util.disable_database_sync()
    context = str(uuid.uuid4())
    cfg = get_config()
    util.handle_output_level(args)
    configuration = util.prepare_configuration(args)
    answerfile_path = cfg.get('report', 'answerfile')
    userchoices_path = cfg.get('report', 'userchoices')
    # NOTE(ivasilev) argparse choices and defaults in enough for validation
    report_schema = args.report_schema

    if os.getuid():
        raise CommandError('This command has to be run under the root user.')
    e = Execution(context=context, kind='preupgrade', configuration=configuration)
    e.store()
    util.archive_logfiles()
    logger = configure_logger('leapp-preupgrade.log')
    os.environ['LEAPP_EXECUTION_ID'] = context

    try:
        repositories = util.load_repositories()
    except LeappError as exc:
        raise CommandError(exc.message)

    workflow = repositories.lookup_workflow('IPUWorkflow')()
    util.warn_if_unsupported(configuration)
    util.process_whitelist_experimental(repositories, workflow, configuration, logger)
    with beautify_actor_exception():
        workflow.load_answers(answerfile_path, userchoices_path)
        until_phase = 'ReportsPhase'
        logger.info('Executing workflow until phase: %s', until_phase)

        # Set the locale, so that the actors parsing command outputs that might be localized will not fail
        os.environ['LANGUAGE'] = 'en_US.UTF-8'
        os.environ['LC_ALL'] = 'en_US.UTF-8'
        os.environ['LANG'] = 'en_US.UTF-8'
        workflow.run(context=context, until_phase=until_phase, skip_dialogs=True)

    logger.info("Answerfile will be created at %s", answerfile_path)
    workflow.save_answers(answerfile_path, userchoices_path)
    util.generate_report_files(context, report_schema)
    report_errors(workflow.errors)
    report_files = util.get_cfg_files('report', cfg)
    log_files = util.get_cfg_files('logs', cfg)
    report_info(context, report_files, log_files, answerfile_path, fail=workflow.failure, errors=workflow.errors)

    if workflow.failure:
        sys.exit(1)


def register(base_command):
    """
        Registers `leapp preupgrade`
    """
    base_command.add_sub(preupgrade)



import re

from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api

# The openssl configuration file
# TODO copied from opensslconfigscanner/libraries/readconf.py
CONFIG = '/etc/pki/tls/openssl.cnf'

LEAPP_COMMENT = '# Modified by leapp during upgrade to RHEL 9\n'
APPEND_STRING = (
    '[provider_sect]\n'
    'default = default_sect\n'
    '##legacy = legacy_sect\n'
    '\n'
    '[default_sect]\n'
    'activate = 1\n'
    '\n'
    '##[legacy_sect]\n'
    '##activate = 1\n'
)


def _add_lines(lines, add):
    """
    Add lines to the list of lines. Breaking possible newlines onto separate items
    """
    for l in add.split("\n"):
        lines.append("{}\n".format(l))
    return lines


class NotFoundException(Exception):
    pass


def _replace(lines, search, replace, comment=None, backup=False, fail_on_error=True):
    """
    Replace pattern with new value with optional backup

    Replace the lines (ignoring leading and trailing whitespace) matching `search` regex
    in the given file (as `lines` collected using readlines()) with the `replace` line optional
    comment is added on line preceding the change.
    """
    res = []
    found = False
    for line in lines:
        if re.search(search, line.strip()):
            if comment:
                res.append(comment)
            if backup:
                res.append("# {}".format(line))
            res = _add_lines(res, replace)
            found = True
        else:
            res.append(line)
    if not found and fail_on_error:
        raise NotFoundException("The pattern {} not found.".format(search))
    return res


def _append(lines, add, comment=None):
    """
    Append a line to the existing list with optional comment
    """
    if comment:
        lines.append(comment)
    return _add_lines(lines, add)


def _modify_file(f, fail_on_error=True):
    """
    Modify the openssl configuration file to accommodate el8toel9 changes
    """
    lines = f.readlines()
    lines = _replace(lines, r"openssl_conf\s*=\s*default_modules",
                     "openssl_conf = openssl_init",
                     LEAPP_COMMENT, True, fail_on_error)
    lines = _replace(lines, r"\[\s*default_modules\s*\]",
                     "[openssl_init]\n"
                     "providers = provider_sect",
                     LEAPP_COMMENT, True, fail_on_error)
    lines = _append(lines, APPEND_STRING, LEAPP_COMMENT)
    f.seek(0)
    f.write(''.join(lines))


def process(openssl_messages):
    """
    Process the changes needed to update configuration file

    Steps:
     * read the file
     * replace the required chunks
     * write the file
    """
    config = next(openssl_messages, None)
    if list(openssl_messages):
        api.current_logger().warning('Unexpectedly received more than one OpenSslConfig message.')
    if not config:
        raise StopActorExecutionError(
            'Could not check openssl configuration', details={'details': 'No OpenSslConfig facts found.'}
        )

    # If the configuration file was not modified, the rpm update will bring the new
    # changes by itself -- do not change the file now.
    if not config.modified:
        return

    # otherwise modify the file as announced in actors/opensslconfigcheck/actor.py
    api.current_logger().debug('Modifying the {}.'.format(CONFIG))
    try:
        with open(CONFIG, 'r+') as f:
            _modify_file(f)
    except (OSError, IOError, NotFoundException) as error:
        api.current_logger().error('Failed to modify the file {}: {} '.format(CONFIG, error))
        reporting.create_report([
            reporting.Title('Could not modify {}: {}'.format(CONFIG, error)),
            reporting.Summary(
                'The original version was kept in place.'
                'The OpenSSL should keep working as expected in most of the cases, but you '
                'might encounter some issues if you need to use custom providers. Consider '
                'updating the configuration manually. For reference, see the openssl.cnf.rpmnew'
            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.SECURITY,
                    reporting.Groups.NETWORK,
                    reporting.Groups.SERVICES
            ]),
            reporting.Groups([
                reporting.Groups.POST
            ]),
            reporting.RelatedResource('package', 'openssl'),
            reporting.RelatedResource('file', '/etc/pki/tls/openssl.cnf')
        ])



from leapp.actors import Actor
from leapp.libraries.actor import checkselinux
from leapp.models import KernelCmdlineArg, Report, SELinuxFacts, SelinuxPermissiveDecision, SelinuxRelabelDecision
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckSelinux(Actor):
    """
    Check SELinux status and produce decision messages for further action.

    Based on SELinux status produces decision messages to relabeling and changing status if
    necessary
    """

    name = 'check_se_linux'
    consumes = (SELinuxFacts,)
    produces = (
        KernelCmdlineArg,
        Report,
        SelinuxPermissiveDecision,
        SelinuxRelabelDecision,
    )
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        checkselinux.process()



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class CPUInfo(Model):
    """
    The model represents information about CPUs.

    The model currently doesn't represent all information about cpus could
    provide on the machine. Just part of them, in case any other attributes
    will be needed, the model can be extended.

    The provided info is aggregated - like from lscpu command. Expecting all
    CPUs are same on the machine (at least for now).
    """

    topic = SystemFactsTopic

    machine_type = fields.Nullable(fields.Integer())
    """
    Specifies machine type if provided.

    This is important for the check of s390x, whether the HW is supported
    by RHEL 8.
    """

    # TODO: regarding possible problems with LOCALE, I am keeping most of
    # parts commented out and focus just on the one particular needed info.
    # architecture = fields.String()
    # """ Architecture of the CPU (e.g. x86_64) """

    # byte_order = fields.StringEnum(['Little Endian', 'Big Endian'])
    # """ Byte order of the CPU: 'Little Endian' or 'Big Endian' """

    flags = fields.List(fields.String(), default=[])
    """ Specifies flags/features of the CPU. """

    # hypervisor = fields.Nullable(fields.String())
    # hypervisor_vendor = fields.Nullable(fields.String())

    # number = fields.Integer()
    # """ Number of CPUs. """

    # vendor_id = fields.Nullable(fields.String())
    # """ ID of vendor of the CPU. """

    # virtualization = fields.Nullable(fields.String())
    # virtualization_type = fields.Nullable(fields.String())



import os

from leapp.actors import Actor
from leapp.libraries.actor import spamassassinconfigread
from leapp.libraries.common.utils import read_file
from leapp.models import DistributionSignedRPM, SpamassassinFacts
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class SpamassassinConfigRead(Actor):
    """
    Reads spamc configuration (/etc/mail/spamassassin/spamc.conf), the
    spamassassin sysconfig file (/etc/sysconfig/spamassassin) and checks
    whether the spamassassin service has been overridden. Produces
    SpamassassinFacts containing the extracted information.
    """

    name = 'spamassassin_config_read'
    consumes = (DistributionSignedRPM,)
    produces = (SpamassassinFacts,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        if spamassassinconfigread.is_processable():
            self.produce(spamassassinconfigread.get_spamassassin_facts(read_func=read_file,
                                                                       listdir=os.listdir))



from leapp.actors import Actor
from leapp.libraries.actor import readopensshconfig
from leapp.models import OpenSshConfig
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class OpenSshConfigScanner(Actor):
    """
    Collect information about the OpenSSH configuration.

    Currently supporting the following options:

     * PermitRootLogin
     * UsePrivilegeSeparation
     * Protocol
     * Ciphers
     * MACs
     * Subsystem sftp

    """

    name = 'read_openssh_config'
    consumes = ()
    produces = (OpenSshConfig, )
    tags = (FactsPhaseTag, IPUWorkflowTag, )

    def process(self):
        readopensshconfig.scan_sshd(self.produce)



from leapp.actors import Actor
from leapp.models import (
    DistributionSignedRPM,
    FilteredRpmTransactionTasks,
    PESRpmTransactionTasks,
    RpmTransactionTasks
)
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class FilterRpmTransactionTasks(Actor):
    """
    Filter RPM transaction events based on installed RPM packages

    In order to calculate a working DNF Upgrade transaction, Leapp can collect data from multiple
    sources and find workarounds for possible problems. This actor will filter all collected
    workarounds and keep only those relevants to current system based on installed packages.
    """

    name = 'check_rpm_transaction_events'
    consumes = (PESRpmTransactionTasks, RpmTransactionTasks, DistributionSignedRPM,)
    produces = (FilteredRpmTransactionTasks,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        installed_pkgs = set()
        for rpm_pkgs in self.consume(DistributionSignedRPM):
            installed_pkgs.update([pkg.name for pkg in rpm_pkgs.items])

        local_rpms = set()
        to_install = set()
        to_remove = set()
        to_keep = set()
        to_upgrade = set()
        modules_to_enable = {}
        modules_to_reset = {}
        for event in self.consume(RpmTransactionTasks, PESRpmTransactionTasks):
            local_rpms.update(event.local_rpms)
            to_install.update(event.to_install)
            to_remove.update(installed_pkgs.intersection(event.to_remove))
            to_keep.update(installed_pkgs.intersection(event.to_keep))
            modules_to_enable.update({'{}:{}'.format(m.name, m.stream): m for m in event.modules_to_enable})
            modules_to_reset.update({'{}:{}'.format(m.name, m.stream): m for m in event.modules_to_reset})

        to_remove.difference_update(to_keep)

        # run upgrade for the rest of RH signed pkgs which we do not have rule for
        to_upgrade = installed_pkgs - (to_install | to_remove)

        self.produce(FilteredRpmTransactionTasks(
            local_rpms=list(local_rpms),
            to_install=list(to_install),
            to_remove=list(to_remove),
            to_keep=list(to_keep),
            to_upgrade=list(to_upgrade),
            modules_to_reset=list(modules_to_reset.values()),
            modules_to_enable=list(modules_to_enable.values())))



#
# Helper functions
#

import re

from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import XorgDrv


def check_drv_and_options(driver, logs):
    regex_driver = re.compile(''.join([driver, '.*DPI set to']))
    regex_options = re.compile(''.join([r'\(\*\*\)', '.*', driver]))
    has_driver = False
    has_options = False

    for line in logs:
        if re.search(regex_driver, line):
            has_driver = True
        if re.search(regex_options, line):
            has_options = True

    if not has_driver:
        return None

    return XorgDrv(driver=driver, has_options=has_options)


def get_xorg_logs_from_journal():
    try:
        output = run(['/usr/bin/journalctl', '/usr/libexec/Xorg', '-o', 'cat', '-b', '0'], split=True)
    except CalledProcessError:
        api.current_logger().debug('No Xorg logs found in journal.')
        return []

    return output['stdout']



import os

import pytest

from leapp import reporting
from leapp.libraries.actor import checkinitramfstasks
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import DracutModule, KernelModule, TargetInitramfsTasks, UpgradeInitramfsTasks
from leapp.utils.report import is_inhibitor


def gen_UIT(modules):
    if not isinstance(modules, list):
        modules = [modules]
    dracut_modules = [DracutModule(name=i[0], module_path=i[1]) for i in modules]
    kernel_modules = [KernelModule(name=i[0], module_path=i[1]) for i in modules]
    return UpgradeInitramfsTasks(include_dracut_modules=dracut_modules, include_kernel_modules=kernel_modules)


def gen_TIT(modules):
    if not isinstance(modules, list):
        modules = [modules]
    dracut_modules = [DracutModule(name=i[0], module_path=i[1]) for i in modules]
    return TargetInitramfsTasks(include_dracut_modules=dracut_modules)


@pytest.mark.parametrize('expected_res,input_msgs,test_msg_type', [
    (
        {},
        [],
        UpgradeInitramfsTasks,
    ),
    (
        {},
        [gen_UIT([('modA', 'pathA'), ('modB', 'pathB')])],
        UpgradeInitramfsTasks,
    ),
    (
        {},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathA')])],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'pathA', 'pathB'}},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathB')])],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'pathA', 'pathB'}},
        [gen_UIT(('modA', 'pathA')), gen_UIT(('modA', 'pathB'))],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'pathA', 'pathB'}},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathB'), ('modB', 'pathC')])],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {os.path.join(checkinitramfstasks.DRACUT_MOD_DIR, 'modA'), 'pathB'}},
        [gen_UIT([('modA', None), ('modA', 'pathB')])],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'pathA', 'pathB'}},
        [gen_TIT([('modA', 'pathA'), ('modA', 'pathB')])],
        TargetInitramfsTasks,
    ),
    (
        {},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathB')])],
        TargetInitramfsTasks,
    ),
])
def test_dracut_conflict_detection(monkeypatch, expected_res, input_msgs, test_msg_type):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=input_msgs))
    res = checkinitramfstasks._detect_modules_conflicts(test_msg_type, 'dracut')
    assert res == expected_res


@pytest.mark.parametrize('expected_res,input_msgs,test_msg_type', [
    (
        {},
        [],
        UpgradeInitramfsTasks,
    ),
    (
        {},
        [gen_UIT([('modA', 'pathA'), ('modB', 'pathB')])],
        UpgradeInitramfsTasks,
    ),
    (
        {},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathA')])],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'pathA', 'pathB'}},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathB')])],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'pathA', 'pathB'}},
        [gen_UIT(('modA', 'pathA')), gen_UIT(('modA', 'pathB'))],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'pathA', 'pathB'}},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathB'), ('modB', 'pathC')])],
        UpgradeInitramfsTasks,
    ),
    (
        {'modA': {'modA (system)', 'pathB'}},
        [gen_UIT([('modA', None), ('modA', 'pathB')])],
        UpgradeInitramfsTasks,
    ),
    (
        {},
        [gen_UIT([('modA', 'pathA'), ('modA', 'pathB')])],
        TargetInitramfsTasks,
    ),
])
def test_kernel_conflict_detection(monkeypatch, expected_res, input_msgs, test_msg_type):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=input_msgs))
    res = checkinitramfstasks._detect_modules_conflicts(test_msg_type, 'kernel')
    assert res == expected_res


def test_report_uit(monkeypatch):
    input_msgs = [gen_UIT([('modA', 'pathA'), ('modA', 'pathB')])]
    sum_msg = "- modA: ['pathA', 'pathB']"
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=input_msgs))
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    checkinitramfstasks.process()
    assert reporting.create_report.called
    assert 'upgrade' in reporting.create_report.report_fields['title']
    assert sum_msg in reporting.create_report.report_fields['summary']
    assert is_inhibitor(reporting.create_report.report_fields)


def test_report_tit(monkeypatch):
    input_msgs = [gen_TIT([('modA', 'pathA'), ('modA', 'pathB')])]
    sum_msg = "- modA: ['pathA', 'pathB']"
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=input_msgs))
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    checkinitramfstasks.process()
    assert reporting.create_report.called
    assert 'target' in reporting.create_report.report_fields['title']
    assert sum_msg in reporting.create_report.report_fields['summary']
    assert is_inhibitor(reporting.create_report.report_fields)


def test_no_conflict(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[]))
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    checkinitramfstasks.process()
    assert not reporting.create_report.called



import pytest

from leapp.models import QuaggaToFrrFacts
from leapp.snactor.fixture import ActorContext


# TODO We can't use caplog here as logs from other processes is
#  hard to capture and caplog not see it.
@pytest.mark.parametrize(
    ("quagga_facts", "active_daemons", "has_report", "msg_in_log"),
    [
        (True, ["babeld"], True, None),
        (True, ["something_else"], False, "babeld not used, moving on"),
        (False, [], False, None),
    ],
)
def test_quaggareport(
    monkeypatch,
    current_actor_context,
    quagga_facts,
    active_daemons,
    has_report,
    msg_in_log,
):
    """Test quaggareport.

    :type current_actor_context:ActorContext
    """
    if quagga_facts:
        current_actor_context.feed(
            QuaggaToFrrFacts(
                active_daemons=active_daemons,
                enabled_daemons=["bgpd", "ospfd", "zebra"],
            )
        )
    current_actor_context.run()
    if has_report:
        assert current_actor_context.messages()[0]["type"] == "Report"
    if msg_in_log:
        assert not current_actor_context.messages()



from leapp.actors import Actor
from leapp.libraries.actor.checkosrelease import check_os_version, skip_check
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckOSRelease(Actor):
    """
    Check if the current RHEL minor version is supported. If not, inhibit the upgrade process.

    This check can be skipped by using the LEAPP_DEVEL_SKIP_CHECK_OS_RELEASE environment variable.
    """

    name = 'check_os_release'
    consumes = ()
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        if not skip_check():
            check_os_version()



import os

from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor.addupgradebootentry import add_boot_entry, fix_grub_config_error
from leapp.models import BootContent, FirmwareFacts, GrubConfigError, TargetKernelCmdlineArgTasks, TransactionDryRun
from leapp.tags import InterimPreparationPhaseTag, IPUWorkflowTag


class AddUpgradeBootEntry(Actor):
    """
    Add new boot entry for Leapp provided initramfs.

    Using new boot entry, Leapp can continue the upgrade process in the initramfs after reboot
    """

    name = 'add_upgrade_boot_entry'
    consumes = (BootContent, GrubConfigError, FirmwareFacts, TransactionDryRun)
    produces = (TargetKernelCmdlineArgTasks,)
    tags = (IPUWorkflowTag, InterimPreparationPhaseTag)

    def process(self):
        for grub_config_error in self.consume(GrubConfigError):
            if grub_config_error.error_detected:
                fix_grub_config_error('/etc/default/grub', grub_config_error.error_type)

        configs = None
        ff = next(self.consume(FirmwareFacts), None)
        if not ff:
            raise StopActorExecutionError(
                'Could not identify system firmware',
                details={'details': 'Actor did not receive FirmwareFacts message.'}
            )

        # related to issue with hybrid BIOS and UEFI images
        # https://bugzilla.redhat.com/show_bug.cgi?id=1667028
        if ff.firmware == 'bios' and os.path.ismount('/boot/efi') and os.path.isfile('/boot/efi/EFI/redhat/grub.cfg'):
            configs = ['/boot/grub2/grub.cfg', '/boot/efi/EFI/redhat/grub.cfg']
        add_boot_entry(configs)



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class FileInfo(Model):
    """
    Various data about a file.

    This model is not supposed to be used as a message directly.
    See e.g. :class:`TrackedSourceFilesInfo` instead.
    """
    topic = SystemInfoTopic

    path = fields.String()
    """
    Canonical path to the file.
    """

    exists = fields.Boolean()
    """
    True if the file is present on the system.
    """

    rpm_name = fields.String(default="")
    """
    Name of the rpm that owns the file. Otherwise empty string if not owned
    by any rpm.
    """

    # NOTE(pstodulk): I have been thinking about the "state"/"modified" field
    # instead. Which could contain enum list, where could be specified what has
    # been changed (checksum, type, owner, ...). But currently we do not have
    # use cases for that and do not want to implement it now. So starting simply
    # with this one.
    is_modified = fields.Boolean()
    """
    True if the checksum of the file has been changed (includes the missing state).

    The field is valid only for a file tracked by rpm - excluding ghost files.
    In such a case the value is always false.
    """


class TrackedFilesInfoSource(Model):
    """
    Provide information about files on the source system explicitly defined
    in the actor to be tracked.

    Search an actor producing this message to discover the list where you
    could add the file into the list to be tracked.

    This particular message is expected to be produced only once by the
    specific actor. Do not produce multiple messages of this model.
    """
    topic = SystemInfoTopic

    files = fields.List(fields.Model(FileInfo), default=[])
    """
    List of :class:`FileInfo`.
    """
